type prs;

# Model proposed in
# Ing-Ray Chen And I.-Ling Yen,
# "Analysis of Probabilistic Error Checking Procedures on Storage systems"
# The Computer Journal, Vol 38, No 5, 1995.

Const
        # rate at which access operations arrive
        lambda = 3.0;
        # rate at which an access operation is served by the system
        mu= 5.0;
        # execution rate at which the errorchecking/recovery procedure is performed
        sigma= 5.0;
        # error rate, either software or hardware-induced
        gamma= 5.0e-7;
        # probability of performing error checking per access,  q in [0,1]
        q= 0.2;
        # queue length
        Q=20;

State
        i : 0 To Q;     # queue length
        j : 0 To 2;     # {j=0 or j=1} no error  {j=2} one error

        @F;             # more then one error occured

Matrix Q
               [i,j]->[i+1,j]   = lambda;
        j==0 : [i,j]->[i-1,j]   = (1-q)*mu;
        j==0 : [i,j]->[i,j+1]   = q*mu;
        j==0 : [i,j]->[i,j+2]   = gamma;
        j==1 : [i,j]->[i-1,j-1] = sigma;
        j==2 : [i,j]->[i,j-1]   = q*mu;
        j==2 : [i,j]->[@F]      = gamma+(1-q)*mu;

Vector R
        i>0 & j==0 : [i,j] = (1-q)*mu;
        i>0 & j==1 : [i,j] = sigma;

Vector P0
        i==0 & j==0 : [i,j] = 1;      # starting state




