import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.awt.event.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class AnalysisRunner extends JFrame implements MRMAnalysisListener {
  JPanel jPanel1 = new JPanel();
  JButton stop = new JButton();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel2 = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel jPanel3 = new JPanel();
  JLabel loverall = new JLabel();
  JProgressBar overallprogress = new JProgressBar();
  Border border1;
  Border border2;
  TitledBorder titledBorder1;
  MRM model;
  JPanel jPanel4 = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel jPanel5 = new JPanel();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel jPanel6 = new JPanel();
  JLabel lsubtask = new JLabel();
  JProgressBar subtaskprogress = new JProgressBar();
  Border border3;
  Border border4;
  TitledBorder titledBorder2;
  JScrollPane jScrollPane1 = new JScrollPane();
  JTextArea messages = new JTextArea();
  Border border5;
  TitledBorder titledBorder3;
  JCheckBox closeatfinish = new JCheckBox();
  BorderLayout borderLayout5 = new BorderLayout();
  JPanel jPanel7 = new JPanel();

  MRMAnalyser myAnalyser = null;
  RunMomAnalysis momAnalysis = null;
  RunDistrAnalysis distrAnalysis = null;
  RunCTfmDistrAnalysis ctfmdistrAnalysis = null;
  boolean stopped = false;
  DataSeries myDS;
  long startTime;
  JPanel jPanel8 = new JPanel();
  GridLayout gridLayout1 = new GridLayout();
  JPanel jPanel9 = new JPanel();
  JPanel jPanel10 = new JPanel();
  JLabel elapsed = new JLabel();
  JLabel jLabel2 = new JLabel();
  JLabel remaining = new JLabel();
  JLabel jLabel4 = new JLabel();

  private int  laststat = 0;
  private long lasttime = 0;

  public AnalysisRunner(MRMAnalyser mrma, DataSeries ds) throws HeadlessException {

//    super (mrma.model.mw, false);

    myDS = ds;
    myAnalyser = mrma;

    Boolean b = (Boolean)((Option)MainWindow.options.get("Close analysis window when finished")).getValue();
    closeatfinish.setSelected(b.booleanValue());

    try {
      model = mrma.model;
      jbInit();
      setSize (350,300);
      Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mine = getSize();
      setLocation ((screen.width-mine.width)/2, (screen.height-mine.height)/2);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void executeMomentAnalysis (double start, double end, int steps, boolean norm, boolean isAR) {
    setVisible (true);
    model.addAnalysisListener(this);
    overallprogress.setValue(0);
    subtaskprogress.setValue(0);
    loverall.setText(" ");
    lsubtask.setText(" ");
    myDS.changing = true;
    momAnalysis = new RunMomAnalysis(this, model, myDS.usedPS, myDS.running, start, end, steps, myDS.cols, norm, isAR);
    startTime = System.currentTimeMillis();
    momAnalysis.start();
    toFront();
  }

  public void executeDistributionAnalysis (double x, double start, double end, int steps, boolean normalized, boolean isAR) {
    setVisible (true);
    model.addAnalysisListener(this);
    overallprogress.setValue(0);
    subtaskprogress.setValue(0);
    loverall.setText(" ");
    lsubtask.setText(" ");
    myDS.changing = true;
    distrAnalysis = new RunDistrAnalysis(this, model, myDS.usedPS, x, start, end, steps, normalized, isAR);
    startTime = System.currentTimeMillis();
    distrAnalysis.start();
    toFront();
  }

  public void executeCTfmDistributionAnalysis (double x, double start, double end, int steps, boolean normalized, int numofmoments) {
    setVisible (true);
    model.addAnalysisListener(this);
    overallprogress.setValue(0);
    subtaskprogress.setValue(0);
    loverall.setText(" ");
    lsubtask.setText(" ");
    myDS.changing = true;
    ctfmdistrAnalysis = new RunCTfmDistrAnalysis(this, model, myDS.usedPS, x, start, end, steps, normalized, numofmoments);
    startTime = System.currentTimeMillis();
    ctfmdistrAnalysis.start();
    toFront();
  }

  public void updateStatus (String overalltxt, int overalltotal, int overallcompl, String subtxt, int subtasktotal, int subtaskcompl, String out) {

    overallprogress.setMaximum(overalltotal);
    overallprogress.setValue(overallcompl);
    loverall.setText(overalltxt);
    subtaskprogress.setMaximum(subtasktotal);
    subtaskprogress.setValue(subtaskcompl);
    lsubtask.setText(subtxt);
    messages.setText(out);
    messages.setCaretPosition(out.length());

    long etime = System.currentTimeMillis() - startTime;

    long hours = etime / 3600000;
    long mins = (etime - hours*3600000) / 60000;
    long secs = (etime - hours*3600000 - mins*60000) / 1000;

    if (hours>0)
      elapsed.setText(""+hours+"h "+mins+"m "+secs+"s");
    else if (mins>0)
      elapsed.setText(""+mins+"m "+secs+"s");
    else
      elapsed.setText(""+secs+"s");

    if ((subtaskcompl==0 || subtasktotal==0) && laststat==0)
      return;

    if (subtaskcompl==0 || subtasktotal==0) {
      remaining.setText("n/a");
      laststat = 0;
      lasttime = etime;
      System.out.println("reset");
    }
    else if (subtaskcompl!=laststat) {

      long ntime = etime - lasttime;
      double ncompl = (double)(subtaskcompl - laststat) / (subtasktotal-laststat);

      long rtime = Math.round( (double) ntime / ncompl);

//      System.out.println("ncompl="+ncompl+"ntime="+ntime+" rtime="+rtime);

      lasttime = etime;
      laststat = subtaskcompl;

      hours = rtime / 3600000;
      mins = (rtime - hours * 3600000) / 60000;
      secs = (rtime - hours * 3600000 - mins * 60000) / 1000;

      if (hours > 0)
        remaining.setText("" + hours + "h " + mins + "m " + secs + "s");
      else if (mins > 0)
        remaining.setText("" + mins + "m " + secs + "s");
      else
        remaining.setText("" + secs + "s");
    }
}

  public void addResults (double[][] newResults) {
    if (newResults!=null && (momAnalysis!=null || distrAnalysis!=null || ctfmdistrAnalysis!=null))
      myDS.addItems(newResults);
  }

  public void finish () {
    toFront();
    if (myDS.table.getRowCount()>0)
      myDS.table.setRowSelectionInterval(0,0);
    stop.setText("Close");
    stop.setToolTipText("Close window");
    long endTime = System.currentTimeMillis();
    messages.append("\nFinished, total time = "+(endTime-startTime)+"ms");
    messages.setCaretPosition(messages.getText().length());
    overallprogress.setMaximum(1);
    overallprogress.setValue(1);
    subtaskprogress.setMaximum(1);
    subtaskprogress.setValue(1);
    myDS.changing = false;
    if (!stopped && closeatfinish.isSelected())
      setVisible (false);
  }

  public void start () {
    setTitle("Running Analysis...");
    stop.setEnabled(true);
  }

  private void jbInit() throws Exception {
    border1 = BorderFactory.createEmptyBorder(4,4,4,4);
    border2 = BorderFactory.createEmptyBorder();
    titledBorder1 = new TitledBorder(new EtchedBorder(EtchedBorder.RAISED,Color.white,new Color(148, 145, 140)),"Current Analysis");
    border3 = BorderFactory.createEmptyBorder(4,4,4,4);
    border4 = new EtchedBorder(EtchedBorder.RAISED,Color.white,new Color(178, 178, 178));
    titledBorder2 = new TitledBorder(border4,"Messages");
    border5 = new EtchedBorder(EtchedBorder.RAISED,Color.white,new Color(148, 145, 140));
    titledBorder3 = new TitledBorder(border5,"Messages");
    this.setTitle("Preparing Analysis...");
    this.addWindowListener(new AnalysisRunner_this_windowAdapter(this));
    this.getContentPane().setLayout(borderLayout1);
    stop.setEnabled(false);
    stop.setToolTipText("Abort analysis");
    stop.setText("Stop Analysis");
    stop.addActionListener(new AnalysisRunner_stop_actionAdapter(this));
    jPanel2.setLayout(borderLayout2);
    jPanel2.setBorder(border1);
    overallprogress.setMaximum(1000);
    jPanel4.setLayout(borderLayout3);
    jPanel5.setLayout(borderLayout4);
    lsubtask.setText(" ");
    loverall.setText(" ");
    jPanel5.setBorder(border3);
    messages.setText("");
    jScrollPane1.setBorder(titledBorder3);
    jScrollPane1.setToolTipText("Messages of the solver");
    subtaskprogress.setMaximum(1000);
    closeatfinish.setToolTipText("When selected, this window will be closed automatically after analysis");
    closeatfinish.setText("Close at finish");
    jPanel1.setLayout(borderLayout5);
    jPanel8.setLayout(gridLayout1);
    gridLayout1.setColumns(2);
    elapsed.setText("0s");
    jLabel2.setText("Time elapsed:");
    jLabel4.setText("Remaining:");
    this.getContentPane().add(jPanel1, BorderLayout.SOUTH);
    jPanel1.add(closeatfinish, BorderLayout.WEST);
    this.getContentPane().add(jPanel2, BorderLayout.NORTH);
    jPanel2.add(jPanel3,  BorderLayout.NORTH);
    jPanel3.add(loverall, null);
    jPanel2.add(overallprogress,  BorderLayout.SOUTH);
    this.getContentPane().add(jPanel4, BorderLayout.CENTER);
    jPanel4.add(jPanel5, BorderLayout.NORTH);
    jPanel5.add(jPanel6,  BorderLayout.CENTER);
    jPanel6.add(lsubtask, null);
    jPanel5.add(subtaskprogress,  BorderLayout.SOUTH);
    jPanel4.add(jScrollPane1, BorderLayout.CENTER);
    jPanel4.add(jPanel8,  BorderLayout.SOUTH);
    jPanel8.add(jPanel9, null);
    jPanel9.add(jLabel2, null);
    jPanel9.add(elapsed, null);
    jPanel8.add(jPanel10, null);
    jPanel10.add(jLabel4, null);
    jPanel10.add(remaining, null);
    jScrollPane1.getViewport().add(messages, null);
    jPanel1.add(jPanel7,  BorderLayout.CENTER);
    jPanel7.add(stop, null);
  }

  void stop_actionPerformed(ActionEvent e) {
    if (model.analyserProcess!=null) {
      stopped = true;
      model.analyserProcess.abort();
      myDS.changing = false;
      if (myDS.dm.getRowCount()>0) {
        int retval = JOptionPane.showConfirmDialog(this, "Delete results computed so far?", "Confirmation", JOptionPane.YES_NO_OPTION);
        if (retval==JOptionPane.YES_OPTION)
          myAnalyser.delDataSeries (myDS);
      }
      else
        myAnalyser.delDataSeries (myDS);
    }
    setVisible (false);
  }

  void this_windowClosing(WindowEvent e) {
    if (model.analyserProcess!=null) {
      stopped = true;
      model.analyserProcess.abort();
      myDS.changing = false;
      if (myDS.dm.getRowCount()>0) {
        int retval = JOptionPane.showConfirmDialog(this, "Delete results computed so far?", "Confirmation", JOptionPane.YES_NO_OPTION);
        if (retval==JOptionPane.YES_OPTION)
          myAnalyser.delDataSeries (myDS);
      }
      else
        myAnalyser.delDataSeries (myDS);
    }
  }

}

class RunMomAnalysis extends Thread {

  public MRM model;
  public MRMParameterSet ps;
  public String running;
  public double start;
  public double end;
  public int steps;
  public int numOfMoms;
  public boolean norm;
  AnalysisRunner ar;
  public boolean ready = false;
  public boolean isAR = true;

  public RunMomAnalysis (AnalysisRunner ar, MRM model, MRMParameterSet ps, String running, double start, double end, int steps, int numOfMoms, boolean norm, boolean isar) {
    isAR = isar;
    this.ar = ar;
    this.model = model;
    this.ps = ps;
    this.running = running;
    this.start = start;
    this.end = end;
    this.steps = steps;
    this.numOfMoms = numOfMoms;
    this.norm = norm;
  }

  public void run () {
    if (isAR)
      model.getARMoments(ps, running, start, end, steps, numOfMoms, norm);
    else
      model.getCTMoments(ps, running, start, end, steps, numOfMoms, norm);
    ready = true;
  }
}

class RunDistrAnalysis extends Thread {

  public MRM model;
  public MRMParameterSet ps;
  public double start;
  public double end;
  public int steps;
  AnalysisRunner ar;
  public boolean ready = false;
  public boolean isAR = true;
  public double x;
  public boolean normalized;

  public RunDistrAnalysis (AnalysisRunner ar, MRM model, MRMParameterSet ps, double x, double start, double end, int steps, boolean normalized, boolean isar) {
    isAR = isar;
    this.ar = ar;
    this.model = model;
    this.ps = ps;
    this.start = start;
    this.end = end;
    this.steps = steps;
    this.x = x;
    this.normalized = normalized;
  }

  public void run () {
    if (isAR)
      model.getARDistribution(ps, x, start, end, steps, normalized);
    else
      model.getCTDistribution(ps, x, start, end, steps, normalized);
    ready = true;
  }
}

class RunCTfmDistrAnalysis extends Thread {

  public MRM model;
  public MRMParameterSet ps;
  public double start;
  public double end;
  public int steps;
  AnalysisRunner ar;
  public boolean ready = false;
  public int numofmoments;
  public double x;
  public boolean normalized;

  public RunCTfmDistrAnalysis (AnalysisRunner ar, MRM model, MRMParameterSet ps, double x, double start, double end, int steps, boolean normalized, int numofmoments) {
    this.numofmoments = numofmoments;
    this.ar = ar;
    this.model = model;
    this.ps = ps;
    this.start = start;
    this.end = end;
    this.steps = steps;
    this.x = x;
    this.normalized = normalized;
  }

  public void run () {

    ((PRSMRM)model).getCTDistributionfm(ps, x, start, end, steps, normalized, numofmoments);
    ready = true;
  }
}


class AnalysisRunner_stop_actionAdapter implements java.awt.event.ActionListener {
  AnalysisRunner adaptee;

  AnalysisRunner_stop_actionAdapter(AnalysisRunner adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.stop_actionPerformed(e);
  }
}

class AnalysisRunner_this_windowAdapter extends java.awt.event.WindowAdapter {
  AnalysisRunner adaptee;

  AnalysisRunner_this_windowAdapter(AnalysisRunner adaptee) {
    this.adaptee = adaptee;
  }
  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}
