
import java.util.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.io.*;
/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class DataSeries extends JPanel {

  MRMAnalyser parent = null;
  int id = -1;
  public String running = "var";
  public String [] ylabels;
  public Vector items;
  public int cols;
  public String quantity = "";
  String modelString = "";
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JLabel jLabel1 = new JLabel();
  BorderLayout borderLayout2 = new BorderLayout();
  JTextField label = new JTextField();
  Border border1;
  JScrollPane jScrollPane1 = new JScrollPane();
  JTable table = new JTable();
  DSDataModel dm;
  MRMParameterSet usedPS;
  JButton params = new JButton();
  boolean momentType;
  boolean changing = true;

  class DSDataModel extends AbstractTableModel {

    DataSeries ds;
    Vector listeners = new Vector ();

    public DSDataModel (DataSeries ds) {
      this.ds = ds;
    }

    public void addTableModelListener(TableModelListener l) {
      listeners.add(l);
    }

    public int getColumnCount() {
      return cols+1;
    }

    public String getColumnName (int columnIndex) {
      if (columnIndex==0)
        return running;
      else
        return ylabels[columnIndex-1];
    }

    public int getRowCount() {
      return items.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
      DataItem item = (DataItem)(items.elementAt(rowIndex));
      if (columnIndex==0)
        return new Double (item.xval);
      else
        return new Double (item.yvals[columnIndex-1]);
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
      return false;
    }

    public void removeTableModelListener (TableModelListener l) {
      listeners.remove(l);
    }

    public void setValueAt (Object aValue, int rowIndex, int columnIndex) {
      double d = Double.parseDouble(aValue.toString());
      DataItem item = (DataItem)(items.elementAt(rowIndex));
      if (columnIndex==0)
        item.xval = d;
      else
        item.yvals[columnIndex-1] = d;
    }

    public void invokeListeners (TableModelEvent e) {
      Iterator i = listeners.iterator();
      while (i.hasNext()) {
        TableModelListener tml = (TableModelListener)i.next();
        tml.tableChanged(e);
      }
    }
  }

  public DataSeries(MRMParameterSet ps, String label, String running, String[] columns) {
    if (ps!=null)
      usedPS = new MRMParameterSet (ps);
    else
      usedPS = null;
    this.running = running;
    this.label.setText(label);
    items = new Vector ();
    cols = columns.length;
    ylabels = new String [cols];
    for (int i=0; i<cols; i++)
      ylabels[i] = columns[i];
    try {
      jbInit();
      params.setEnabled(usedPS!=null);
      dm = new DSDataModel (this);
      table.setModel(dm);
      table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public DataSeries(String descr) {

    BufferedReader reader = new BufferedReader (new StringReader (descr));
    try {
      jbInit();

    try{
      String line = reader.readLine();
      while (line!=null) {

        if (line.startsWith("# Running parameter:")) {
          running = line.substring(21).trim();
        }
        else if (line.startsWith("# Label:")) {
          label.setText(line.substring(9));
        }
        else if (line.startsWith("# Value")) {
          usedPS = new MRMParameterSet ();
          line = reader.readLine();
          while (line!=null && line.startsWith("#\t")) {
            StringTokenizer stk = new StringTokenizer (line, "# =\t\n");
            String n = stk.nextToken();
            String v = stk.nextToken();
            usedPS.addParameter(new Parameter (n, Double.parseDouble(v)));
            line = reader.readLine();
          }
          continue;
        }
        else if (line.startsWith("# Model Description:")) {
          modelString = "";
          line = reader.readLine();
          while (line!=null && line.startsWith("# -")) {
            modelString = modelString + line.substring(4) + "\n";
            line = reader.readLine();
          }
          continue;
        }
        else if (line.startsWith("# Model name:")) {

        }
        else if (line.startsWith("# ")) { // parse column names
          StringTokenizer stk = new StringTokenizer (line.substring(2), "\t\n");
          int num = stk.countTokens();
          cols = num - 1;
          items = new Vector ();
          ylabels = new String [num-1];
          running = stk.nextToken();
          for (int i=0; i<num-1; i++)
            ylabels[i] = stk.nextToken();
          line = reader.readLine();
          while (line!=null) {
            stk = new StringTokenizer (line, " \t\n");
            if (stk.countTokens()<num)
              break;
            DataItem item = new DataItem ();
            item.xval = Double.parseDouble(stk.nextToken());
            item.yvals = new double[num-1];
            for (int i=0; i<num-1; i++)
              item.yvals[i] = Double.parseDouble(stk.nextToken());
            items.add(item);
            line = reader.readLine();
          }
          break;
        }
        line = reader.readLine();
      }
      reader.close();
    }
    catch (IOException ex) {
    }

      params.setEnabled(usedPS!=null);
      dm = new DSDataModel (this);
      table.setModel(dm);
      table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void refresh () {
    dm.invokeListeners(new TableModelEvent(dm));
  }

  public void addItem (double x, double[] y) {
    DataItem item = new DataItem ();
    item.xval = x;
    item.yvals = new double[cols];
    for (int i=0; i<cols; i++)
      item.yvals[i] = y[i];
    items.add(item);
    dm.invokeListeners(new TableModelEvent (dm, items.indexOf(item)));
  }

  public void addItems (double[][] val) {
    for (int j=0; j<val.length; j++) {
      DataItem item = new DataItem();
      item.xval = val[j][0];
      item.yvals = new double[cols];
      for (int i = 0; i < cols; i++)
        item.yvals[i] = val[j][i+1];
      if (val[j].length>cols+1)
        item.opt1 = val[j][cols+1];
      else
        item.opt1 = 0;
      if (val[j].length>cols+2)
        item.opt2 = val[j][cols+2];
      else
        item.opt2 = 1e200;
      items.add(item);
    }
    dm.invokeListeners(new TableModelEvent(dm));
  }

  public String exportString () {

    StringWriter swriter = new StringWriter();
    PrintWriter writer = new PrintWriter (swriter);

    writer.println("# Model name: "+parent.model.getName());
    writer.println("# Label: "+label.getText());
//    if (momentType) {
      writer.println("# Running parameter: "+running);
//    }
    if (usedPS!=null) {
      writer.println("# Value of model parameters:");
      Iterator it = usedPS.parameters.iterator();
      while (it.hasNext()) {
        Parameter p = (Parameter)it.next();
        writer.println("#\t"+p.name+" = "+p.value);
      }
    }
    if (modelString!=null & modelString.length()>0) {
      writer.println("# Model Description:");
      StringTokenizer stk = new StringTokenizer (modelString, "\n");
      while (stk.hasMoreTokens())
        writer.println("# - "+stk.nextToken());
    }
    writer.print("# ");
    writer.print("# "+running);
    for (int i=0; i<ylabels.length; i++)
      writer.print("\t"+ylabels[i]);
    writer.println();
    for (int i=0; i<dm.getRowCount(); i++) {
      for (int j=0; j<dm.getColumnCount(); j++) {
        if (j>0)
          writer.print('\t');
        writer.print(dm.getValueAt(i, j).toString());
      }
      writer.println();
    }
    writer.close();
    return swriter.toString();
  }

  private void jbInit() throws Exception {
    border1 = BorderFactory.createEmptyBorder(2,4,2,4);
    this.setLayout(borderLayout1);
    jLabel1.setText("Label:");
    jPanel1.setLayout(borderLayout2);
    label.setText("");
    jPanel1.setBorder(border1);
    borderLayout2.setHgap(4);
    borderLayout2.setVgap(4);
    params.setText("Parameters used...");
    params.addActionListener(new DataSeries_params_actionAdapter(this));
    this.add(jPanel1, BorderLayout.NORTH);
    jPanel1.add(jLabel1,  BorderLayout.WEST);
    jPanel1.add(label, BorderLayout.CENTER);
    jPanel1.add(params,  BorderLayout.EAST);
    this.add(jScrollPane1, BorderLayout.CENTER);
    jScrollPane1.getViewport().add(table, null);
  }

  void params_actionPerformed(ActionEvent e) {

    String summary = "<HTML> <BODY bgcolor=%BG%> <H3> "+label.getText()+"</H3><HR>";
    summary += "Running variable: <B>"+running+"</B><BR>";
    if (usedPS!=null && usedPS.parameters.size()>0) {
      summary += "Values of the parameters:";
      summary += "<TABLE border=0 width=100% cellspacing=0><TR><TD><TABLE width=100% border=0 cellspacing=0>";
      Iterator i = usedPS.parameters.iterator();
      String color;
      boolean even = true;
      while (i.hasNext()) {
        Parameter item = (Parameter)i.next();
        if (even)
          color = "%TABLE1%";
        else
          color = "%TABLE2%";
        summary += "<TR bgcolor="+color+"><TD align=left>"+item.name+" = </TD><TD align=left>"+item.value+"</TD></TR>";
        even = !even;
      }
      summary += "</TABLE></TR></TD></TABLE>";
    }

    if (modelString.length()>0) {
    summary += "<BR><B>Model Description:</B><PRE>"+modelString+"</PRE>";
    }
    summary += "</BODY></HTML>";
    HTMLDialog htmld = new HTMLDialog ("Parameters", 250, 300);
    htmld.showString (ColorChanger.changeColor(summary));
  }
}

class DataItem {
  public double xval;
  public double[] yvals;
  public double opt1;
  public double opt2;
}

class DataSeries_params_actionAdapter implements java.awt.event.ActionListener {
  DataSeries adaptee;

  DataSeries_params_actionAdapter(DataSeries adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.params_actionPerformed(e);
  }
}
