import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;
import java.math.*;
import javax.swing.event.*;
import java.beans.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class MRM extends JInternalFrame implements DocumentListener {

  public CommandRunner analyserProcess = null;

  MRMType type;
  String shortname;
  String fullname;
  String description;
  boolean isRaw;
  MRMAnalysisListener listener;
  JPanel editorPanel = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  BorderLayout borderLayout2 = new BorderLayout();
  Border border1;
  TitledBorder titledBorder1;
  JPanel buttonPanel = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JButton analysis = new JButton();
  JPanel jPanel2 = new JPanel();
  JButton showDescr = new JButton();
  JTabbedPane tabbed = new JTabbedPane();
  JScrollPane jScrollPane1 = new JScrollPane();

  Vector matrices = new Vector ();
  Vector vectors = new Vector ();
  JTextArea modelDef = new JTextArea();

  String tmpdir = "";
  JPanel jPanel3 = new JPanel();
  JButton check = new JButton();

  MainWindow mw = null;

  boolean dirty = false;

  public MRM (MRMType t, String shname, String fname, boolean israw) {
    super("Edit " + shname,
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable
    long rnd = (long)Math.floor(Math.random()*1000000.0);
    tmpdir = "tmp/" + shname+ Long.toString(rnd);
    File d = new File (tmpdir);
    d.mkdir();
    shortname = shname;
    fullname = fname;
    isRaw = israw;
    type = t;
    try {
      jbInit();
      if (isRaw) {
        editorPanel.add(tabbed, BorderLayout.CENTER);
      }
      else {
        editorPanel.add(jScrollPane1,  BorderLayout.CENTER);
        jScrollPane1.getViewport().add(modelDef, null);
        modelDef.getDocument().addDocumentListener(this);
        showDescr.setVisible(false);
        check.setVisible(false);
      }
      setSize (400,400);
      setDefaultCloseOperation (JInternalFrame.DO_NOTHING_ON_CLOSE);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }



  public void addMatrix (UMatrix um, String hint) {

    matrices.add(um);
    JPanel tabpanel = new JPanel ();
    tabpanel.setLayout(new BorderLayout ());
    JTable table = new JTable (um);
    JScrollPane jScrollPane = new JScrollPane ();
    tabpanel.add(jScrollPane,  BorderLayout.CENTER);
    jScrollPane.getViewport().add(table, null);
    table.setToolTipText(hint);
    tabbed.add(tabpanel, um.name);

    JPanel editButtons = new JPanel ();
    JButton add = new JButton ("Add Line");
    JButton rem = new JButton ("Remove Line");
    JButton read = new JButton ("Import from File");
    add.addActionListener(new LineAction (table, um, 0));
    rem.addActionListener(new LineAction (table, um, 1));
    read.addActionListener(new LineAction (table, um, 2));
    editButtons.add(add);
    editButtons.add(rem);
    editButtons.add(read);

    tabpanel.add(editButtons, BorderLayout.SOUTH);
  }

  public void addVector (UVector um, String hint) {

    vectors.add(um);
    JPanel tabpanel = new JPanel ();
    tabpanel.setLayout(new BorderLayout ());
    JTable table = new JTable (um);
    JScrollPane jScrollPane = new JScrollPane ();
    tabpanel.add(jScrollPane,  BorderLayout.CENTER);
    jScrollPane.getViewport().add(table, null);
    table.setToolTipText(hint);
    tabbed.add(tabpanel, um.name);

    JPanel editButtons = new JPanel ();
    JButton add = new JButton ("Add Line");
    JButton rem = new JButton ("Remove Line");
    JButton read = new JButton ("Import from File");
    add.addActionListener(new LineAction (table, um, 0));
    rem.addActionListener(new LineAction (table, um, 1));
    read.addActionListener(new LineAction (table, um, 2));
    editButtons.add(add);
    editButtons.add(rem);
    editButtons.add(read);

    tabpanel.add(editButtons, BorderLayout.SOUTH);
  }

  public MRMType getType () {
    return type;
  }

  public String  getName () {
    return shortname;
  }

  public String  getFullFileName () {
    return fullname;
  }

  public String  getDescription () {
    if (!isRaw) {
      description = null;
      BufferedReader reader = new BufferedReader (new StringReader (modelDef.getText()));
      try {
        String line = reader.readLine();
        while (line != null) {
          line = line.trim();
          if (line.startsWith("//--")) {
            if (description==null)
              description = line.substring(4);
            else
              description += "\n"+line.substring(4);
          }
          else
            break;
          line = reader.readLine();
        }
      }
      catch (IOException ex) {}
    }
    return description;
  }


  public void setDescription (String descr) {
    description = descr;
    setDirty (true);
  }

  public boolean isRaw () {
    return isRaw;
  }

  public void load (String fname)  {

    try {
      BufferedReader file = new BufferedReader(new FileReader(fname));
      String line = file.readLine();
      String model = "";
      while (line!=null) {
        model += line + "\n";
        line = file.readLine();
      }
      loadModelFromString (model);

      fullname = fname;
      shortname = new File (fname).getName();
      setTitle ("Edit " + shortname);
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error loading file", JOptionPane.ERROR_MESSAGE);
    }
    setDirty (false);
  }

  public void loadModelFromString (String model) {
    try {
      if (isRaw) {
        BufferedReader file = new BufferedReader(new StringReader(model));
        UMatrix um = null;
        UVector uv = null;
        String line = file.readLine();
        String delim = new String (" \n\r\t,;:(){}[]-_><=");
        int linenum = 1;
        int rstate = 0; // =0: nothing; =1: inside description; =2: inside matrix; =3: inside vector
        while (line!=null) {
          StringTokenizer tk = new StringTokenizer (line, delim);
          int toknum = 1;
          try {
            if (tk.hasMoreTokens() == false) {
              if (rstate==1)
                description += "\n";
              linenum++;
              line = file.readLine();
              continue;
            }
            String first = tk.nextToken();
            if (rstate == 0) {
              if (first.equalsIgnoreCase("type")) {
                linenum++;
                line = file.readLine();
                continue;
              }
              else if (first.equalsIgnoreCase("description")) {
                description = null;
                rstate = 1;
              }
              else if (first.equalsIgnoreCase("matrix")) {
                rstate = 2;
                String name = tk.nextToken();
                Iterator i = matrices.iterator();
                while (i.hasNext()) {
                  UMatrix m = (UMatrix) (i.next());
                  if (m.name.equalsIgnoreCase(name)) {
                    um = m;
                    break;
                  }
                }
                if (um == null) {
                  JOptionPane.showMessageDialog(this,
                                                "In line " + linenum +
                                                ": Unknown matrix name '" +
                                                name + "'!",
                                                "Error reading file",
                                                JOptionPane.ERROR_MESSAGE);
                  break;
                }
              }
              else if (first.equalsIgnoreCase("vector")) {
                rstate = 3;
                String name = tk.nextToken();
                Iterator i = vectors.iterator();
                while (i.hasNext()) {
                  UVector v = (UVector) (i.next());
                  if (v.name.equalsIgnoreCase(name)) {
                    uv = v;
                    break;
                  }
                }
                if (uv == null) {
                  JOptionPane.showMessageDialog(this,
                                                "In line " + linenum +
                                                ": Unknown vector name '" +
                                                name + "'!",
                                                "Error reading file",
                                                JOptionPane.ERROR_MESSAGE);
                  break;
                }
              }
              else {
                JOptionPane.showMessageDialog(this,
                                              "In line " + linenum +
                    ": Unexpected token \n(expected 'matrix', 'vector', or 'description')",
                                              "Error reading file",
                                              JOptionPane.ERROR_MESSAGE);
                break;
              }
            }
            else if (first.equalsIgnoreCase("end"))
              rstate = 0;
            else if (rstate == 1) {
              if (description==null)
                description = line;
              else
                description = description + "\n" + line;
            }
            else if (rstate == 2) {
              // parse um
              int stfrom = Integer.parseInt(first);
              toknum++;
              int stto = Integer.parseInt(tk.nextToken());
              toknum++;
              double strate = Double.parseDouble(tk.nextToken());
              um.addLine(stfrom, stto, strate);
            }
            else if (rstate == 3) {
              // parse uv
              int state = Integer.parseInt(first);
              toknum++;
              double val = Double.parseDouble(tk.nextToken());
              toknum++;
              uv.addLine(state, val);
            }
          }
          catch (NoSuchElementException ex1) {
            JOptionPane.showMessageDialog(this, "In line "+linenum+": "+ex1.toString(),"Error reading file", JOptionPane.ERROR_MESSAGE);
            break;
          }
          catch (NumberFormatException ex2) {
            JOptionPane.showMessageDialog(this, "In line "+linenum+", token "+toknum+": "+ex2.toString(),"Error reading file", JOptionPane.ERROR_MESSAGE);
            break;
          }
          linenum++;
          line = file.readLine();
        }
      }
      else {
        String mdef = "";
        BufferedReader file = new BufferedReader(new StringReader(model));
        String line = file.readLine();
        while (line != null) {
          mdef = mdef + line + "\n";
          line = file.readLine();
        }
        modelDef.setText(mdef);
        modelDef.setCaretPosition(0);
      }
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error loading file", JOptionPane.ERROR_MESSAGE);
    }
  }

  public String saveModelToString () {

    StringWriter swriter = new StringWriter ();
    PrintWriter file = new PrintWriter (swriter);
    if (isRaw) {
      file.println("type " + type.getMnemonic());
      file.println();
      file.println("description");
      file.println(description);
      file.println("end");
      file.println();
      Iterator i = matrices.iterator();
      while (i.hasNext()) {
        UMatrix um = (UMatrix)i.next();
        file.println("matrix " + um.name);
        Iterator ifrom = um.from.iterator();
        Iterator ito   = um.to.iterator();
        Iterator irate = um.rate.iterator();
        while (ifrom.hasNext()) {
          file.print((Integer)(ifrom.next()));
          file.print(" -> ");
          file.print((Integer)(ito.next()));
          file.print(" : ");
          file.println((Double)(irate.next()));
        }
        file.println("end");
        file.println();
      }
      i = vectors.iterator();
      while (i.hasNext()) {
        UVector uv = (UVector)i.next();
        file.println("vector " + uv.name);
        Iterator state = uv.state.iterator();
        Iterator val = uv.val.iterator();
        while (state.hasNext()) {
          file.print( (Integer) (state.next()));
          file.print(" : ");
          file.println( (Double) (val.next()));
        }
        file.println("end");
        file.println();
      }
    }
    else {
      file.print(modelDef.getText());
    }
    return swriter.toString();
  }

  public void save (String fname) {
    try {
      PrintStream file = new PrintStream (new FileOutputStream(fname));
      file.print(saveModelToString ());
      fullname = fname;
      shortname = new File (fname).getName();
      setTitle ("Edit " + shortname);
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error saving file", JOptionPane.ERROR_MESSAGE);
    }
    setDirty (false);
  }

  public MRMParameterSet getParameterSet () {
    MRMParameterSet ps = new MRMParameterSet ();
    return ps;
  }

  public String checkRaw () {

    String error = null;

    Iterator mi = matrices.iterator();
    while (mi.hasNext()) {
      UMatrix m = (UMatrix)mi.next();
      error = m.check ();
      if (error!=null)
        return error;
    }

    Iterator vi = vectors.iterator();
    while (vi.hasNext()) {
      UVector v = (UVector)vi.next();
      error = v.check ();
      if (error!=null)
        return error;
    }

    return null;
  }

  public boolean isDirty () {
    if (isRaw) {
      Iterator i = matrices.iterator();
      while (i.hasNext())
        if (((UMatrix)i.next()).isDirty ())
          return true;
      i = vectors.iterator();
      while (i.hasNext())
        if (((UVector)i.next()).isDirty ())
          return true;
      return false;
    }
    else
      return dirty;
  }

  public void setDirty (boolean d) {
    if (isRaw) {
      Iterator i = matrices.iterator();
      while (i.hasNext())
        ((UMatrix)i.next()).setDirty (d);
      i = vectors.iterator();
      while (i.hasNext())
        ((UVector)i.next()).setDirty (d);
    }
    else
      dirty = d;
  }

  public String workspaceString () {

    String wstring = "$$$ Model type="+type.getMnemonic()+" \""+shortname+"\" \""+fullname+"\"";
    if (isRaw)
      wstring += " raw\n";
    else
      wstring += " formalizm\n";
    wstring += saveModelToString ();
    return wstring;
  }


  public void changedUpdate (DocumentEvent ev) {
    dirty = true;
  }
  public void insertUpdate (DocumentEvent ev) {
    dirty = true;
  }
  public void removeUpdate (DocumentEvent ev) {
    dirty = true;
  }

  public String checkModel () { return null; }
  public void getARMoments (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) { return; }
  public void getCTMoments (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) { return; }
  public void getARDistribution (MRMParameterSet p, double t, double start, double end, int steps, boolean normalized) { return;}
  public void getCTDistribution (MRMParameterSet p, double w, double start, double end, int steps, boolean normalized) { return; }
  public String getStateSpaceHTMLString () { return ""; }
  public void showStateSpace () { }

  public void addAnalysisListener (MRMAnalysisListener l) {
    listener = l;
  }

  public MRM() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    border1 = new EtchedBorder(EtchedBorder.RAISED,Color.white,new Color(148, 145, 140));
    titledBorder1 = new TitledBorder(border1,"Model Delfinition");
    this.getContentPane().setLayout(borderLayout1);
    editorPanel.setLayout(borderLayout2);
    editorPanel.setBorder(titledBorder1);
    buttonPanel.setAlignmentX((float) 0.5);
    buttonPanel.setLayout(borderLayout3);
    analysis.setEnabled(false);
    analysis.setVisible(false);
    analysis.setText("Go to Analysis Window");
    showDescr.setText("Show/Edit Description...");
    showDescr.addActionListener(new MRM_showDescr_actionAdapter(this));
    this.addInternalFrameListener(new MRM_this_internalFrameAdapter(this));
    check.setText("Check Model");
    check.addActionListener(new MRM_check_actionAdapter(this));
    this.getContentPane().add(editorPanel, BorderLayout.CENTER);
    this.getContentPane().add(buttonPanel,  BorderLayout.SOUTH);
    buttonPanel.add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(analysis, null);
    buttonPanel.add(jPanel2,  BorderLayout.WEST);
    jPanel2.add(showDescr, null);
    buttonPanel.add(jPanel3,  BorderLayout.EAST);
    jPanel3.add(check, null);
  }

  void this_internalFrameClosed(InternalFrameEvent e) {
    File d = new File (tmpdir);
    File[] f = d.listFiles();
    if (f!=null) {
      for (int i = 0; i < f.length; i++)
        f[i].delete();
    }
    d.delete();
    MRMAnalyser myAnalyser = mw.getAnalyser(this);
    if (myAnalyser!=null)
      myAnalyser.setVisible(false);
  }

  public String toString () {
    return shortname;
  }

  void check_actionPerformed(ActionEvent e) {
    String s = checkModel();
    if (s==null)
      JOptionPane.showMessageDialog(this, getName()+": OK", "No error found", JOptionPane.INFORMATION_MESSAGE);
    else
      JOptionPane.showMessageDialog(this, getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
  }

  void this_internalFrameClosing(InternalFrameEvent e) {
    boolean isdirty = isDirty ();
    boolean hasanalyser = mw.getAnalyser(this)!=null;
    if (isdirty) {
      int answer = JOptionPane.showConfirmDialog(this, "Are you sure to close the model?\nModel changed, unsaved modifications will be lost!",
                                                 "Closing Model",
                                                 JOptionPane.YES_NO_CANCEL_OPTION,
                                                 JOptionPane.QUESTION_MESSAGE);
      if (answer != JOptionPane.YES_OPTION) {
        return;
      }
    }
    if (hasanalyser) {
      int answer = JOptionPane.showConfirmDialog(this, "Are you sure to close the model?\n The analyser window will be closed, too!",
                                                 "Closing Model",
                                                 JOptionPane.YES_NO_CANCEL_OPTION,
                                                 JOptionPane.QUESTION_MESSAGE);
      if (answer != JOptionPane.YES_OPTION) {
        return;
      }
    }
    dispose ();
  }

  void showDescr_actionPerformed(ActionEvent e) {
    EditDescription ed = new EditDescription (getDescription());
    ed.setVisible(true);
    if (ed.description!=null)
      setDescription(ed.description);
  }
}

class LineAction implements ActionListener {

  UMatrix myUM = null;
  UVector myUV = null;
  int action = 0; //=0: add; =1: remove, =2: read
  JTable table;

  public LineAction (JTable tab, UMatrix um, int a) {
    myUM = um;
    action = a;
    table = tab;
  }

  public LineAction (JTable tab, UVector uv, int a) {
    myUV = uv;
    action = a;
    table = tab;
  }

  public void actionPerformed (ActionEvent ae) {
    if (myUM != null)
      switch (action) {
        case 0:
          myUM.addLine();
          break;
        case 1:
          myUM.delLine(table.getSelectedRow());
          break;
        case 2:
          myUM.load();
          break;
      }
    if (myUV != null)
      switch (action) {
        case 0:
          myUV.addLine();
          break;
        case 1:
          myUV.delLine(table.getSelectedRow());
          break;
        case 2:
          myUV.load();
          break;
      }
    table.invalidate();
    ((JPanel)(table.getParent().getParent().getParent())).revalidate ();
  }
}

class MRM_this_internalFrameAdapter extends javax.swing.event.InternalFrameAdapter {
  MRM adaptee;

  MRM_this_internalFrameAdapter(MRM adaptee) {
    this.adaptee = adaptee;
  }
  public void internalFrameClosed(InternalFrameEvent e) {
    adaptee.this_internalFrameClosed(e);
  }
  public void internalFrameClosing(InternalFrameEvent e) {
    adaptee.this_internalFrameClosing(e);
  }
}

class MRM_check_actionAdapter implements java.awt.event.ActionListener {
  MRM adaptee;

  MRM_check_actionAdapter(MRM adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.check_actionPerformed(e);
  }
}

class MRM_showDescr_actionAdapter implements java.awt.event.ActionListener {
  MRM adaptee;

  MRM_showDescr_actionAdapter(MRM adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.showDescr_actionPerformed(e);
  }
}
