
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class MRMAnalyser extends JInternalFrame {

  MRM model;

  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JButton addARMom = new JButton();
  JButton remSeries = new JButton();
  JButton appendSeries = new JButton();
  JPanel jPanel2 = new JPanel();
  Border border1;
  TitledBorder titledBorder1;
  JTabbedPane resultPanel = new JTabbedPane();
  BorderLayout borderLayout2 = new BorderLayout();
  JButton addARDistr = new JButton();
  JButton addCTMom = new JButton();
  JPanel jPanel3 = new JPanel();
  JPanel jPanel4 = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JButton addCTDistr = new JButton();
  int num = 1;
  GridLayout gridLayout1 = new GridLayout();
  JButton apprDistr = new JButton();
  JButton export = new JButton();
  JPanel jPanel5 = new JPanel();
  JButton ctdfromarm = new JButton();
  public MRMAnalyser (MRM m) {
    super("Analyse " + m.getName(),
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable
    model = m;
    try {
      jbInit();
      addARMom.setEnabled(model.type.canComputeARMoms());
      addARDistr.setEnabled(model.type.canComputeARDistr());
      addCTMom.setEnabled(model.type.canComputeCTMoms());
      addCTDistr.setEnabled(model.type.canComputeCTDistr());
      ctdfromarm.setEnabled(model instanceof PRSMRM);
      setSize (450,400);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public DataSeries[] getDataSeries () {
    DataSeries[] ds = new DataSeries[resultPanel.getComponentCount()];
    for (int i=0; i<ds.length; i++)
      ds[i] = (DataSeries)resultPanel.getComponentAt(i);
    return ds;
  }

  public void addDataSeries (DataSeries ds) {
    resultPanel.add(ds, ""+num+".");
    ds.parent = this;
    ds.id = num;
    resultPanel.setSelectedIndex(resultPanel.getComponentCount()-1);
    num++;
    remSeries.setEnabled(true);
    export.setEnabled(true);
  }

  private void jbInit() throws Exception {
    border1 = BorderFactory.createEmptyBorder();
    titledBorder1 = new TitledBorder(new EtchedBorder(EtchedBorder.RAISED,Color.white,new Color(148, 145, 140)),"Data Series");
    this.getContentPane().setLayout(borderLayout1);
    addARMom.setToolTipText("Compute moments of the accumulated reward");
    addARMom.setText("Add AR Moments");
    addARMom.addActionListener(new MRMAnalyser_addARMom_actionAdapter(this));
    remSeries.setEnabled(false);
    remSeries.setToolTipText("Remove the currently selected data series");
    remSeries.setText("Remove Series");
    remSeries.addActionListener(new MRMAnalyser_remSeries_actionAdapter(this));
    appendSeries.setVisible(false);
    appendSeries.setToolTipText("Append Current Data Series with New Points");
    appendSeries.setText("Append");
    jPanel2.setBorder(titledBorder1);
    jPanel2.setLayout(borderLayout2);
    addARDistr.setToolTipText("Compute distribution of the accumulated reward");
    addARDistr.setText("Add AR Distribution");
    addARDistr.addActionListener(new MRMAnalyser_addARDistr_actionAdapter(this));
    addCTMom.setToolTipText("Compute moments of the completion time");
    addCTMom.setText("Add CT Moments");
    addCTMom.addActionListener(new MRMAnalyser_addCTMom_actionAdapter(this));
    jPanel1.setLayout(borderLayout3);
    addCTDistr.setToolTipText("Compute distribution of the completion time");
    addCTDistr.setText("Add CT Distribution");
    addCTDistr.addActionListener(new MRMAnalyser_addCTDistr_actionAdapter(this));
    jPanel4.setLayout(gridLayout1);
    gridLayout1.setColumns(4);
    apprDistr.setEnabled(false);
    apprDistr.setToolTipText("Estimate distribution based on moments in the selected line");
    apprDistr.setText("Approx. Distribution");
    apprDistr.addActionListener(new MRMAnalyser_apprDistr_actionAdapter(this));
    resultPanel.addChangeListener(new MRMAnalyser_resultPanel_changeAdapter(this));
    export.setEnabled(false);
    export.setToolTipText("Save data in the current tab into a file");
    export.setText("Export Series");
    export.addActionListener(new MRMAnalyser_export_actionAdapter(this));
    ctdfromarm.setText("CT distribution estimation using AR moments");
    ctdfromarm.addActionListener(new MRMAnalyser_ctdfromarm_actionAdapter(this));
    jPanel4.add(addARMom, null);
    jPanel4.add(addARDistr, null);
    jPanel4.add(addCTDistr, null);
    jPanel1.add(jPanel4,  BorderLayout.NORTH);
    jPanel4.add(addCTMom, null);
    jPanel1.add(jPanel3, BorderLayout.SOUTH);
    jPanel3.add(apprDistr, null);
    jPanel3.add(appendSeries, null);
    jPanel3.add(export, null);
    jPanel3.add(remSeries, null);
    jPanel1.add(jPanel5, java.awt.BorderLayout.CENTER);
    jPanel5.add(ctdfromarm);
    this.getContentPane().add(jPanel1, BorderLayout.SOUTH);
    this.getContentPane().add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(resultPanel,  BorderLayout.CENTER);
  }

  void addARMom_actionPerformed(ActionEvent e) {
    String s = model.checkModel();
    if (s==null) {
      NewMomentSeries nms = new NewMomentSeries(this, true);
      nms.setVisible(true);
    }
    else {
      JOptionPane.showMessageDialog(this, model.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
  }

  void addCTMom_actionPerformed(ActionEvent e) {
    String s = model.checkModel();
    if (s==null) {
      NewMomentSeries nms = new NewMomentSeries(this, false);
      nms.setVisible(true);
    }
    else {
      JOptionPane.showMessageDialog(this, model.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
  }

  void addARDistr_actionPerformed(ActionEvent e) {
    String s = model.checkModel();
    if (s==null) {
      NewDistribution nd = new NewDistribution(this, true);
      nd.setVisible(true);
    }
    else {
      JOptionPane.showMessageDialog(this, model.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
  }

  void addCTDistr_actionPerformed(ActionEvent e) {
    String s = model.checkModel();
    if (s==null) {
      NewDistribution nd = new NewDistribution(this, false);
      nd.setVisible(true);
    }
    else {
      JOptionPane.showMessageDialog(this, model.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
  }

  void remSeries_actionPerformed(ActionEvent e) {
    int current = resultPanel.getSelectedIndex();
    if (current>=0) {
      String title = resultPanel.getTitleAt(current);
      int selected = JOptionPane.showConfirmDialog(this,
          "Results in data series (" + title + ") will be lost! Continue?",
          "Warning!", JOptionPane.YES_NO_CANCEL_OPTION);
      if (selected == JOptionPane.YES_OPTION)
        resultPanel.remove (current);
    }
  }

  void delDataSeries (DataSeries ds) {
    resultPanel.remove(ds);
    if (resultPanel.getTabCount()==0) {
      remSeries.setEnabled(false);
      export.setEnabled(false);
    }
  }

  void resultPanel_stateChanged(ChangeEvent e) {

    Object current = resultPanel.getSelectedComponent();
    if (current!=null && ((DataSeries)current).momentType)
      apprDistr.setEnabled(true);
    else
      apprDistr.setEnabled(false);
    if (current!=null)
      remSeries.setEnabled(!((DataSeries)current).changing);
  }

  void apprDistr_actionPerformed(ActionEvent e) {

    Object current = resultPanel.getSelectedComponent();
    if (current!=null && ((DataSeries)current).momentType) {
      DataSeries ds = (DataSeries)current;
      if (ds.table.getSelectedRow()>=0) {
        ApproxDistrib ad = new ApproxDistrib (this, ds);
        ad.setVisible(true);
      }
    }

  }

  void export_actionPerformed(ActionEvent e) {

    Object current = resultPanel.getSelectedComponent();
    if (current!=null) {
      DataSeries ds = (DataSeries)current;
      JFileChooser fc = new JFileChooser("models/");
      if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {

        try {
          PrintWriter writer = new PrintWriter(new FileOutputStream(fc.
              getSelectedFile()));
          writer.print(ds.exportString());
          writer.close();
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this,
                                        "Cannot export data series\n(" +
                                        ex.toString() + ")",
                                        "Error during export!",
                                        JOptionPane.ERROR_MESSAGE);
        }
      }
    }
  }

  public String workspaceString () {

    String wstring = "$$$ Analyse \"" + model.shortname + "\"\n";
    Component[] components = resultPanel.getComponents();
    for (int i=0; i<components.length; i++) {
      DataSeries ds = (DataSeries)components[i];
      wstring += "$$ DataSeries " + resultPanel.getTitleAt(i)+" moments=";
      if (ds.momentType==true)
        wstring += "yes\n";
      else
        wstring += "no\n";
      wstring += (ds.exportString());
    }

    return wstring;
  }

  public void importFromString (String istring) {
    try {
      BufferedReader reader = new BufferedReader(new StringReader(istring));
      String line = reader.readLine();
      while (line != null) {
        StringBuffer buffer = new StringBuffer();
        String cline = "";
        while (line != null && line.startsWith("$$") == false)
          line = reader.readLine();
        if (line == null)
          break;
        cline = line;
        line = reader.readLine();
        while (line != null && line.startsWith("$$") == false) {
          buffer.append(line);
          buffer.append("\n");
          line = reader.readLine();
        }
        if (cline.startsWith("$$ DataSeries")) {
          StringTokenizer stk = new StringTokenizer (cline, " ");
          String token = stk.nextToken();
          token = stk.nextToken () ;
          String title = stk.nextToken(" .");
          num = Integer.parseInt(title);
          stk = new StringTokenizer (cline, "=");
          token = stk.nextToken () ;
          String smom = stk.nextToken();
          DataSeries ds = new DataSeries (buffer.toString());
          ds.momentType = smom.equals("yes");
          resultPanel.add(ds, ""+num+".");
          ds.parent = this;
          ds.id = num;
          resultPanel.setSelectedIndex(resultPanel.getComponentCount()-1);
          num++;
          remSeries.setEnabled(true);
          export.setEnabled(true);
        }
      }
      reader.close();
    }
    catch (IOException ex) {

    }
  }

  public void ctdfromarm_actionPerformed(ActionEvent e) {
    String s = model.checkModel();
    if (s==null) {
      NewCTfmDistribution nd = new NewCTfmDistribution(this);
      nd.setVisible(true);
    }
    else {
      JOptionPane.showMessageDialog(this, model.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
  }
}

class MRMAnalyser_addARMom_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_addARMom_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.addARMom_actionPerformed(e);
  }
}

class MRMAnalyser_addCTMom_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_addCTMom_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.addCTMom_actionPerformed(e);
  }
}

class MRMAnalyser_remSeries_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_remSeries_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.remSeries_actionPerformed(e);
  }
}

class MRMAnalyser_resultPanel_changeAdapter implements javax.swing.event.ChangeListener {
  MRMAnalyser adaptee;

  MRMAnalyser_resultPanel_changeAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void stateChanged(ChangeEvent e) {
    adaptee.resultPanel_stateChanged(e);
  }
}

class MRMAnalyser_apprDistr_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_apprDistr_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.apprDistr_actionPerformed(e);
  }
}

class MRMAnalyser_export_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_export_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.export_actionPerformed(e);
  }
}

class MRMAnalyser_addARDistr_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_addARDistr_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.addARDistr_actionPerformed(e);
  }
}

class MRMAnalyser_addCTDistr_actionAdapter implements java.awt.event.ActionListener {
  MRMAnalyser adaptee;

  MRMAnalyser_addCTDistr_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.addCTDistr_actionPerformed(e);
  }
}

class MRMAnalyser_ctdfromarm_actionAdapter
    implements ActionListener {
  private MRMAnalyser adaptee;
  MRMAnalyser_ctdfromarm_actionAdapter(MRMAnalyser adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.ctdfromarm_actionPerformed(e);
  }
}
