
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.event.*;
import java.io.*;
import javax.swing.border.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class MainWindow extends JFrame implements ActionListener {


  ImageIcon newIcon = new ImageIcon ("images/new.gif");
  ImageIcon openIcon = new ImageIcon ("images/open.gif");
  ImageIcon saveIcon = new ImageIcon ("images/save.gif");
  ImageIcon anIcon = new ImageIcon ("images/cblue2.gif");
  ImageIcon diagIcon = new ImageIcon ("images/diag.gif");
  JDesktopPane desktop = new JDesktopPane ();

  // Possible Look & Feels
  private static final String maclf      =
          "com.sun.java.swing.plaf.mac.MacLookAndFeel";
  private static final String metallf    =
          "javax.swing.plaf.metal.MetalLookAndFeel";
  private static final String motiflf    =
          "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
  private static final String windowslf  =
          "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
  private static final String gtklf  =
          "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
  private static final String kunststofflf =
          "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
  private static final String liquidlf =
          "com.birosoft.liquid.LiquidLookAndFeel";
  private static final String xplf =
          "com.stefankrause.xplookandfeel.XPLookAndFeel";
  // The current Look & Feel
  private static String currentLookAndFeel = metallf;

  JToolBar jToolBar1 = new JToolBar();
  JButton saveButton = new JButton();
  JButton newButton = new JButton();
  JMenuBar jMenuBar1 = new JMenuBar();
  JMenu file = new JMenu();
  JMenuItem menuNew = new JMenuItem();
  JMenuItem menuOpen = new JMenuItem();
  JMenuItem menuSave = new JMenuItem();
  JMenuItem menuSaveAs = new JMenuItem();
  JButton openButton = new JButton();
  JPanel statusBar = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JMenuItem menuExit = new JMenuItem();
  JMenu analyse = new JMenu();
  JMenu windows = new JMenu();
  JMenu jMenu2 = new JMenu();
  JMenu jMenu3 = new JMenu();
  JMenuItem openGraph = new JMenuItem();
  JMenu modelmenu = new JMenu();
  public int numofmodels = 0;

  Vector mrmTypes = new Vector ();
  static Hashtable options = new Hashtable ();

  JButton anButton = new JButton();
  JMenu mSkinlf = new JMenu();
  JMenuItem mMetallf = new JMenuItem();
  JMenuItem mWindowslf = new JMenuItem();
  JMenuItem mMotiflf = new JMenuItem();
  JMenuItem mMaclf = new JMenuItem();
  JMenuItem mGtklf = new JMenuItem();
  JMenuItem mKunststofflf = new JMenuItem();
  JMenuItem mLiquidlf = new JMenuItem();
  JMenuItem mXPlf = new JMenuItem();
  JButton diagButton = new JButton();
  JMenuItem statespace = new JMenuItem();
  JMenuItem preferences = new JMenuItem();
  Border border1;
  JMenuItem checkModel = new JMenuItem();
  JMenuItem help = new JMenuItem();
  OptionsDialog odialog = null;
  JMenuItem drawStateSpace = new JMenuItem();
  JMenuItem menuSaveW = new JMenuItem();
  JMenuItem menuOpenW = new JMenuItem();
  JLabel jLabel1 = new JLabel();

  public MainWindow() throws HeadlessException {
    try {
      findMRMTypes ();
      fillLFMenu ();
      fillOptions ();
      jbInit();
      setLookAndFeel (kunststofflf);
      ColorChanger cc = new ColorChanger ();
      options.put(cc.color.getName(), cc.color);

      odialog = new OptionsDialog (this);
      try {
        odialog.loadOptions();
      }
      catch (IOException ex) {}
      setSize (920, 600);
      setExtendedState (JFrame.MAXIMIZED_BOTH);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void findMRMTypes () {
    mrmTypes.add (new PRSType ());
    mrmTypes.add (new PILType ());
    mrmTypes.add (new SOType ());
    mrmTypes.add (new SOPILType ());
  }

  void fillOptions () {

    Option o1 = new Option ("Close analysis window when finished", Option.BOOL, new Boolean (false));
    options.put(o1.getName(), o1);
    o1.addDescription("<html><body bgcolor=%BG%><h2>Close analysis window when finished</h2>If selected, analysis window will be closed right after analysis finishes. Disable this option if you want to read the messages of the analyser or check the elapsed time.</body></html>");

  }

  public String [] getWindowTitles () {
    JInternalFrame [] frames = desktop.getAllFrames();
    String [] titles = new String [frames.length];
    for (int i=0; i<frames.length; i++)
      titles[i] = frames[i].getTitle();
    return titles;
  }

  public Vector getModels () {
    JInternalFrame [] frames = desktop.getAllFrames();
    Vector models = new Vector ();
    for (int i=0; i<frames.length; i++)
      if (frames[i].getTitle().startsWith("Edit "))
        models.add(frames[i]);
    return models;
  }

  public MRMAnalyser getAnalyser (MRM model) {
    JInternalFrame [] frames = desktop.getAllFrames();
    MRMAnalyser an = null;
    for (int i=0; i<frames.length; i++)
      if (frames[i].getTitle().startsWith("Analyse ")) {
        MRMAnalyser aa = (MRMAnalyser)(frames[i]);
        if (aa.model == model)
          an = aa;
      }
    return an;
  }

  public void openAnalyserWindow (MRM model) {
    MRMAnalyser an = getAnalyser (model);
    if (an==null) {
      JInternalFrame [] frames = desktop.getAllFrames();
      int a = 0;
      for (int i=0; i<frames.length; i++)
        if (frames[i].getTitle().startsWith("Analyse "))
          a++;
      an = new MRMAnalyser(model);
      desktop.add(an);
      an.setLocation(desktop.getWidth()/2+a*30, a*30);
      an.setVisible(true);
    }
    else {
      desktop.setSelectedFrame(an);
    }
  }

  public void actionPerformed (ActionEvent ae) {

    if (ae.getActionCommand().equals("New")) {
      numofmodels++;
      NewMRMDialog newMRMDialog = new NewMRMDialog (mrmTypes, "Untitled-"+numofmodels);
      newMRMDialog.setVisible (true);
      if (newMRMDialog.selected!=null) {
        MRM mrm = newMRMDialog.selected.getInstance(newMRMDialog.name, newMRMDialog.name, !newMRMDialog.useFormalizm);
        mrm.setDescription(newMRMDialog.description);
        mrm.setLocation((numofmodels-1)*30, (numofmodels-1)*30);
        desktop.add(mrm);
        mrm.setVisible(true);
        mrm.mw = this;
        try {
          mrm.setSelected(true);
        } catch (java.beans.PropertyVetoException e) {}
      }
    }
    else if (ae.getActionCommand().equals("Exit")) {
      System.exit(0);
    }
    else if (ae.getActionCommand().equals("Save") || ae.getActionCommand().equals("Save As...")) {

      JInternalFrame selected = desktop.getSelectedFrame();
      if (selected instanceof MRM) {
        MRM model = (MRM)selected;
        String fname = model.getFullFileName();
        if (ae.getActionCommand().equals("Save As...")) {
          JFileChooser chooser = new JFileChooser("models");
          int returnVal = chooser.showSaveDialog (this);
          if(returnVal == JFileChooser.APPROVE_OPTION) {
            File fsel = chooser.getSelectedFile();
            if (fsel.getName().indexOf(".")==-1) {
              if (model.isRaw)
                model.save(fsel.getAbsolutePath()+".rrm");
              else
                model.save(fsel.getAbsolutePath()+".frm");
            }
            else
              model.save(fsel.getAbsolutePath());
          }
        }
        else
          model.save(fname);
      }
    }
    else if (ae.getActionCommand().equals("Open")) {
      JFileChooser chooser = new JFileChooser("models");
      FRMFileFilter frm = new FRMFileFilter ();
      RRMFileFilter rrm = new RRMFileFilter ();
      chooser.addChoosableFileFilter(frm);
      chooser.addChoosableFileFilter(rrm);
      chooser.setFileFilter(frm);
      int returnVal = chooser.showOpenDialog (this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
        numofmodels++;
        String fname = chooser.getSelectedFile().getAbsolutePath();
        String shname = chooser.getSelectedFile().getName();

        boolean isRaw = false;
        if (shname.endsWith(".rrm"))
          isRaw = true;
        else if (shname.endsWith(".frm"))
          isRaw = false;
        else {
          Object[] possibleValues = { "Raw Description", "Description with formalizm" };
          Object selectedValue = JOptionPane.showInputDialog(this, "Model description type could not be obtained from the file extension.\n Please select the way you defined the model:", "Select description type", JOptionPane.INFORMATION_MESSAGE, null, possibleValues, possibleValues[0]);
          if (selectedValue!=null)
            isRaw = selectedValue.toString().equals("Raw Description");
          else
            return;
        }
        // first detect model type
        String mnemonic = null;
        try {
          BufferedReader file = new BufferedReader(new FileReader(fname));
          String delim = new String (" \n\r\t,;:(){}[]-_><=");
          String line = file.readLine();
          while (line != null) {
            StringTokenizer tk = new StringTokenizer (line, delim);
            if (tk.hasMoreTokens()) {
              if (tk.nextToken().equals("type")) {
                mnemonic = tk.nextToken();
              }
              break;
            }
            line = file.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this,"Error reading file", "Error reading file", JOptionPane.ERROR_MESSAGE);
        }
        MRMType mrmType = null;
        if (mnemonic==null) {
          AskMRMType ask = new AskMRMType (mrmTypes);
          ask.setVisible(true);
          mrmType = ask.selected;
          if (mrmType==null)
            return;
          mnemonic = mrmType.getMnemonic();
        }
        else {
          Iterator i = mrmTypes.iterator();
          while (i.hasNext()) {
            MRMType mrmt = (MRMType) (i.next());
            if (mrmt.getMnemonic().equalsIgnoreCase(mnemonic)) {
              mrmType = mrmt;
              break;
            }
          }
        }
        if (mrmType!=null) {
          MRM model = mrmType.getInstance(shname, fname, isRaw);
          model.load(fname);
          desktop.add(model);
          model.setLocation( (numofmodels - 1) * 30, (numofmodels - 1) * 30);
          model.setVisible(true);
          model.mw = this;
          try {
            model.setSelected(true);
          }
          catch (java.beans.PropertyVetoException e) {}
        }
        else
        if (mnemonic!=null)
          JOptionPane.showMessageDialog(this,"Unknown MRM type: "+mnemonic,"File open error!", JOptionPane.ERROR_MESSAGE);
        else
          JOptionPane.showMessageDialog(this,"Unknown MRM type! No 'type = ...' line found!","File open error!", JOptionPane.ERROR_MESSAGE);
      }
    }
    else if (ae.getActionCommand().equals("Analyse")) {
      String name = ( (JMenuItem) ae.getSource()).getText();
      JInternalFrame[] frames = desktop.getAllFrames();
      for (int i = 0; i < frames.length; i++)
        if (frames[i].getTitle().startsWith("Edit " + name)) {
          openAnalyserWindow ((MRM)frames[i]);
          break;
        }
    }
    else if (ae.getActionCommand().equals("AnalyseCurrent")) {
      JInternalFrame frame = desktop.getSelectedFrame();
      if (frame!=null && frame instanceof MRM)
          openAnalyserWindow ((MRM)frame);
    }
    else if (ae.getActionCommand().equals("lookf")) {
      if (ae.getSource()==mMetallf)
        setLookAndFeel (metallf);
      else if (ae.getSource()==mWindowslf)
        setLookAndFeel (windowslf);
      else if (ae.getSource()==mMaclf)
        setLookAndFeel (maclf);
      else if (ae.getSource()==mMotiflf)
        setLookAndFeel (motiflf);
      else if (ae.getSource()==mGtklf)
        setLookAndFeel (gtklf);
      else if (ae.getSource()==mKunststofflf)
        setLookAndFeel (kunststofflf);
      else if (ae.getSource()==mLiquidlf)
        setLookAndFeel (liquidlf);
      else if (ae.getSource()==mXPlf)
        setLookAndFeel (xplf);
    }
    else if (ae.getActionCommand().equals("Window")) {
      String title = ((JMenuItem)ae.getSource()).getText();
      JInternalFrame[] frames = desktop.getAllFrames();
      for (int i = 0; i < frames.length; i++)
        if (frames[i].getTitle().equals(title)) {
          try {
            frames[i].setSelected(true);
          }
          catch (java.beans.PropertyVetoException e) {
            e.printStackTrace();
          }
          break;
        }
    }
    else if (ae.getActionCommand().equals("Grapheditor")) {
      DiagramEditor de = new DiagramEditor (this);
      desktop.add(de);
      de.setVisible(true);
    }
    else if (ae.getActionCommand().equals("statespace")) {
      MRM mrm = (MRM)(desktop.getSelectedFrame());
      String sshtml = mrm.getStateSpaceHTMLString();
      HTMLDialog htmld = new HTMLDialog ("States", 150, 400);
      htmld.showString (ColorChanger.changeColor(sshtml));
    }
    else if (ae.getActionCommand().equals("Preferences")) {
      odialog.setVisible(true);
    }
    else if (ae.getActionCommand().equals("CheckModel")) {
      MRM mrm = (MRM)(desktop.getSelectedFrame());
      String s = mrm.checkModel();
      if (s==null)
        JOptionPane.showMessageDialog(this, mrm.getName()+": OK", "No error found", JOptionPane.INFORMATION_MESSAGE);
      else
        JOptionPane.showMessageDialog(this, mrm.getName()+": "+s, "Error in model definition", JOptionPane.ERROR_MESSAGE);
    }
    else if (ae.getActionCommand().equals("HelpIndex")) {
      HelpDialog hd = new HelpDialog ();
      hd.setVisible(true);
    }
    else if (ae.getActionCommand().equals("drawstates")) {
      MRM mrm = (MRM)(desktop.getSelectedFrame());
      if (mrm==null)
        return;
      if (mrm.type.canDrawStateSpace()==false) {
        JOptionPane.showMessageDialog(this, "This feature is not available for this mrm type!",
                                      "Error",
                                      JOptionPane.ERROR_MESSAGE);
        return;
      }
      mrm.showStateSpace();
    }
    else if (ae.getActionCommand().equals("SaveW")) {
      JFileChooser chooser = new JFileChooser("models");
      chooser.setFileFilter (new MRWFileFilter());
      int returnVal = chooser.showSaveDialog (this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
        File fsel = chooser.getSelectedFile();
        String fname = fsel.getAbsolutePath();
        if (fsel.getName().indexOf(".")==-1)
          fname += ".mws";
        saveWorkSpace (fname);
      }
    }
    else if (ae.getActionCommand().equals("OpenW")) {
      JFileChooser chooser = new JFileChooser("models");
      chooser.setFileFilter (new MRWFileFilter());
      int returnVal = chooser.showOpenDialog (this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
        File fsel = chooser.getSelectedFile();
        String fname = fsel.getAbsolutePath();
        openWorkSpace (fname);
      }
    }
  }

  void saveWorkSpace (String fname) {

    try {
      PrintWriter writer = new PrintWriter (new FileWriter (fname));
      JInternalFrame [] frames = desktop.getAllFrames();
      Vector models = new Vector ();
      for (int i=0; i<frames.length; i++)
        if (frames[i] instanceof MRM)
          writer.println(((MRM)frames[i]).workspaceString());
      for (int i=0; i<frames.length; i++)
        if (frames[i] instanceof MRMAnalyser)
          writer.println(((MRMAnalyser)frames[i]).workspaceString());
      for (int i=0; i<frames.length; i++)
        if (frames[i] instanceof DiagramEditor)
          writer.println(((DiagramEditor)frames[i]).workspaceString());
      writer.println("$$$ End");
      writer.close();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,"Error writing file", "Error writing file", JOptionPane.ERROR_MESSAGE);
    }
  }

  void openWorkSpace (String fname) {

    try {
      BufferedReader reader = new BufferedReader (new FileReader (fname));
      String line = reader.readLine();
      while (line!=null && line.startsWith("$$$ End")==false) {
        StringBuffer buffer = new StringBuffer ();
        String cline = "";
        while (line!=null && line.startsWith("$$$")==false)
          line = reader.readLine();
        if (line==null)
          break;
        cline = line;
        line = reader.readLine();
        while (line!=null && line.startsWith("$$$")==false) {
          buffer.append(line);
          buffer.append("\n");
          line = reader.readLine();
        }
        if (cline.startsWith("$$$ Model")) {
          StringTokenizer stk = new StringTokenizer (cline, "=");
          String token = stk.nextToken();
          String mnemonic = stk.nextToken("= ");
          String shname = stk.nextToken("\"");
          shname = stk.nextToken("\"");
          String flname = stk.nextToken("\"");
          flname = stk.nextToken("\"");
          String dtype = stk.nextToken(" ");
          dtype = stk.nextToken(" \n");
          boolean isRaw;
          if (dtype.equals("formalizm"))
            isRaw = false;
          else
            isRaw = true;
          MRMType mrmType = null;
          Iterator i = mrmTypes.iterator();
          while (i.hasNext()) {
            MRMType mrmt = (MRMType) (i.next());
            if (mrmt.getMnemonic().equalsIgnoreCase(mnemonic)) {
              mrmType = mrmt;
              break;
            }
          }
          if (mrmType!=null) {
            numofmodels++;
            MRM model = mrmType.getInstance(shname, flname, isRaw);
            model.loadModelFromString(buffer.toString());
            desktop.add(model);
            model.setLocation( (numofmodels - 1) * 30, (numofmodels - 1) * 30);
            model.setVisible(true);
            model.mw = this;
            try {
              model.setSelected(true);
            }
            catch (java.beans.PropertyVetoException e) {}
          }
        }
        else if (cline.startsWith("$$$ Analyse")) {
          StringTokenizer stk = new StringTokenizer (cline, "\"");
          String token = stk.nextToken();
          String shname = stk.nextToken();
          MRM mrm = null;
          JInternalFrame[] frames = desktop.getAllFrames();
          for (int i = 0; i < frames.length; i++)
            if (frames[i].getTitle().startsWith("Edit " + shname)) {
              mrm = (MRM)frames[i];
              openAnalyserWindow (mrm);
              break;
            }
          if (mrm!=null) {
            MRMAnalyser mrma = getAnalyser (mrm);
            mrma.importFromString (buffer.toString());
          }
        }
        else if (cline.startsWith("$$$ DiagramEditor")) {
          DiagramEditor de = new DiagramEditor (this);
          desktop.add(de);
          de.setVisible(true);
          de.importFromString(cline, buffer.toString());
        }
      }
      reader.close();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,"Error reading file", "Error reading file", JOptionPane.ERROR_MESSAGE);
    }
  }

  public static void main(String[] args) throws HeadlessException {
    System.out.println("Starting MRMSolve2...");
    MainWindow mainWindow1 = new MainWindow();
    mainWindow1.setVisible (true);
  }
  private void jbInit() throws Exception {
    border1 = BorderFactory.createEmptyBorder(2,4,2,4);
    this.setJMenuBar(jMenuBar1);
    this.setTitle("MRMSolve2");
    this.addWindowListener(new MainWindow_this_windowAdapter(this));
    this.getContentPane().setLayout(borderLayout1);
    saveButton.setBorder(border1);
    saveButton.setToolTipText("Save model");
    saveButton.setActionCommand("Save");
    saveButton.setIcon(saveIcon);
    saveButton.setText("");
    file.setText("File");
    file.addMenuListener(new MainWindow_file_menuAdapter(this));
    menuNew.setActionCommand("New");
    menuNew.setIcon(newIcon);
    menuNew.setMnemonic('0');
    menuNew.setText("New");
    menuNew.setAccelerator(javax.swing.KeyStroke.getKeyStroke('N', java.awt.event.KeyEvent.CTRL_MASK, false));
    menuOpen.setActionCommand("Open");
    menuOpen.setIcon(openIcon);
    menuOpen.setText("Open...");
    menuOpen.setAccelerator(javax.swing.KeyStroke.getKeyStroke('O', java.awt.event.KeyEvent.CTRL_MASK, false));
    menuSave.setIcon(saveIcon);
    menuSave.setText("Save");
    menuSave.setAccelerator(javax.swing.KeyStroke.getKeyStroke('S', java.awt.event.KeyEvent.CTRL_MASK, false));
    menuSaveAs.setText("Save As...");
    openButton.setBorder(border1);
    openButton.setToolTipText("Open model from file");
    openButton.setActionCommand("Open");
    openButton.setIcon(openIcon);
    openButton.setText("");
    newButton.setBorder(border1);
    newButton.setToolTipText("Define new model");
    newButton.setActionCommand("New");
    newButton.setBorderPainted(false);
    newButton.setIcon(newIcon);
    newButton.setRolloverEnabled(true);
    newButton.setText("");
    menuExit.setText("Exit");
    menuExit.setAccelerator(javax.swing.KeyStroke.getKeyStroke('Q', java.awt.event.KeyEvent.CTRL_MASK, false));
    analyse.setText("Analyse");
    analyse.addMenuListener(new MainWindow_analyse_menuAdapter(this));
    windows.setText("Windows");
    windows.addMenuListener(new MainWindow_windows_menuAdapter(this));
    jMenu2.setText("Options");
    jMenu3.setText("Help");
    openGraph.setActionCommand("Grapheditor");
    openGraph.setIcon(diagIcon);
    openGraph.setText("Open Graph Editor");
    openGraph.setAccelerator(javax.swing.KeyStroke.getKeyStroke('G', java.awt.event.KeyEvent.CTRL_MASK, false));
    modelmenu.setAlignmentX((float) 0.5);
    modelmenu.setIcon(anIcon);
    modelmenu.setText("Analyse Model");
    modelmenu.addMenuListener(new MainWindow_modelmenu_menuAdapter(this));
    anButton.setBorder(border1);
    anButton.setToolTipText("Analyse model");
    anButton.setActionCommand("AnalyseCurrent");
    anButton.setIcon(anIcon);
    anButton.setText("");
    mSkinlf.setActionCommand("lookf");
    mSkinlf.setText("Look & Feel");
    mMetallf.setActionCommand("lookf");
    mMetallf.setText("Metal");
    mWindowslf.setActionCommand("lookf");
    mWindowslf.setText("Windows");
    mMotiflf.setActionCommand("lookf");
    mMotiflf.setText("Motif");
    mMaclf.setActionCommand("lookf");
    mMaclf.setText("Mac");
    mGtklf.setActionCommand("lookf");
    mGtklf.setText("Gtk");
    mKunststofflf.setActionCommand("lookf");
    mKunststofflf.setText("Kunststoff");
    mLiquidlf.setActionCommand("lookf");
    mLiquidlf.setText("Liquid");
    mXPlf.setActionCommand("lookf");
    mXPlf.setText("WinXP");
    diagButton.setBorder(border1);
    diagButton.setActionCommand("Grapheditor");
    diagButton.setIcon(diagIcon);
    statespace.setActionCommand("statespace");
    statespace.setText("Show State Space");
    preferences.setText("Preferences");
    checkModel.setActionCommand("CheckModel");
    checkModel.setText("Check Model");
    checkModel.setAccelerator(javax.swing.KeyStroke.getKeyStroke('K', java.awt.event.KeyEvent.CTRL_MASK, false));
    help.setActionCommand("HelpIndex");
    help.setText("Help Index...");
    help.setAccelerator(javax.swing.KeyStroke.getKeyStroke('H', java.awt.event.KeyEvent.CTRL_MASK, false));
    drawStateSpace.setActionCommand("drawstates");
    drawStateSpace.setText("Draw State Space");
    menuSaveW.setActionCommand("SaveW");
    menuSaveW.setText("Save Workspace");
    menuOpenW.setActionCommand("OpenW");
    menuOpenW.setText("Open Workspace");
    jLabel1.setFont(new java.awt.Font("Dialog", Font.BOLD | Font.ITALIC, 11));
    jLabel1.setText("MRMSolve Special Edition for Dani");
    this.getContentPane().add(jToolBar1, BorderLayout.NORTH);
    jToolBar1.add(newButton, null);
    jToolBar1.add(openButton, null);
    jToolBar1.add(saveButton, null);
    jToolBar1.addSeparator();
    jToolBar1.add(anButton, null);
    jToolBar1.addSeparator();
    jToolBar1.add(diagButton, null);
    jToolBar1.setRollover(true);
    jMenuBar1.add(file);
    jMenuBar1.add(analyse);
    jMenuBar1.add(windows);
    jMenuBar1.add(jMenu2);
    jMenuBar1.add(jMenu3);
    file.add(menuNew);
    file.add(menuOpen);
    file.add(menuSave);
    file.add(menuSaveAs);
    file.addSeparator();
    file.add(menuSaveW);
    file.add(menuOpenW);
    file.addSeparator();
    file.add(menuExit);
    this.getContentPane().add(statusBar, BorderLayout.SOUTH);
    statusBar.add(jLabel1);
    this.getContentPane().add(desktop, BorderLayout.CENTER);
    analyse.add(modelmenu);
    analyse.addSeparator();
    analyse.add(checkModel);
    analyse.add(statespace);
    analyse.add(drawStateSpace);
    analyse.addSeparator();
    analyse.add(openGraph);
    jMenu2.add(preferences);
    jMenu2.add(mSkinlf);
    mSkinlf.add(mMetallf);
    mSkinlf.add(mWindowslf);
    mSkinlf.add(mMotiflf);
    mSkinlf.add(mMaclf);
    mSkinlf.add(mGtklf);
    mSkinlf.add(mKunststofflf);
    mSkinlf.add(mLiquidlf);
    mSkinlf.add(mXPlf);
    jMenu3.add(help);
    menuNew.addActionListener(this);
    menuOpen.addActionListener(this);
    menuSave.addActionListener(this);
    menuSaveAs.addActionListener(this);
    menuExit.addActionListener(this);
    newButton.addActionListener(this);
    openButton.addActionListener(this);
    saveButton.addActionListener(this);
    mMetallf.addActionListener(this);
    mWindowslf.addActionListener(this);
    mMotiflf.addActionListener(this);
    mMaclf.addActionListener(this);
    mGtklf.addActionListener(this);
    mKunststofflf.addActionListener(this);
    mLiquidlf.addActionListener(this);
    mXPlf.addActionListener(this);
    anButton.addActionListener(this);
    openGraph.addActionListener(this);
    diagButton.addActionListener(this);
    statespace.addActionListener(this);
    preferences.addActionListener(this);
    checkModel.addActionListener(this);
    help.addActionListener(this);
    drawStateSpace.addActionListener(this);
    menuOpenW.addActionListener(this);
    menuSaveW.addActionListener(this);
  }

  void this_windowClosing(WindowEvent e) {
    System.exit(0);
  }


  void windows_menuSelected(MenuEvent e) {
    windows.removeAll();
    String [] titles = getWindowTitles();
    for (int i=0; i<titles.length; i++) {
      JMenuItem m = new JMenuItem (titles[i]);
      m.setActionCommand("Window");
      m.addActionListener(this);
      windows.add(m);
    }
  }

  void modelmenu_menuSelected(MenuEvent e) {
    modelmenu.removeAll();
    Vector models = getModels();
    Iterator i = models.iterator();
    while (i.hasNext()) {
      JMenuItem m = new JMenuItem (i.next().toString());
      m.setActionCommand("Analyse");
      m.addActionListener(this);
      modelmenu.add(m);
    }
  }

  protected void fillLFMenu () {
    mMetallf.setEnabled(isAvailableLookAndFeel(metallf));
    mMotiflf.setEnabled(isAvailableLookAndFeel(motiflf));
    mWindowslf.setEnabled(isAvailableLookAndFeel(windowslf));
    mMaclf.setEnabled(isAvailableLookAndFeel(maclf));
    mGtklf.setEnabled(isAvailableLookAndFeel(gtklf));
    mKunststofflf.setEnabled(isAvailableLookAndFeel(kunststofflf));
    mLiquidlf.setEnabled(isAvailableLookAndFeel(liquidlf));
    mXPlf.setEnabled(isAvailableLookAndFeel(xplf));
  }

  protected boolean isAvailableLookAndFeel(String laf) {
      try {
          Class lnfClass = Class.forName(laf);
          LookAndFeel newLAF = (LookAndFeel)(lnfClass.newInstance());
          return newLAF.isSupportedLookAndFeel();
      } catch(Exception e) { // If ANYTHING weird happens, return false
          return false;
      }
  }
  public void setLookAndFeel(String laf) {
      if(currentLookAndFeel != laf) {
          currentLookAndFeel = laf;
          try {
            UIManager.setLookAndFeel(currentLookAndFeel);
            SwingUtilities.updateComponentTreeUI(this);
            if (odialog!=null)
              odialog.updateUI ();
          } catch (Exception ex) {
                System.out.println("Failed loading L&F: " + currentLookAndFeel);
                System.out.println(ex);
            }
      }
  }

  void analyse_menuSelected(MenuEvent e) {
    JInternalFrame selected = desktop.getSelectedFrame();
    if (selected!=null && (selected instanceof MRM)) {
      modelmenu.setEnabled(true);
      checkModel.setEnabled(true);
      if (((MRM)selected).isRaw)
        statespace.setEnabled(false);
      else
        statespace.setEnabled(true);
    }
    else {
      modelmenu.setEnabled(false);
      checkModel.setEnabled(false);
      statespace.setEnabled(false);
    }
  }

  void file_menuSelected(MenuEvent e) {
    JInternalFrame selected = desktop.getSelectedFrame();
    if (selected!=null && (selected instanceof MRM)) {
      menuSave.setEnabled(true);
      menuSaveAs.setEnabled(true);
    }
    else {
      menuSave.setEnabled(false);
      menuSaveAs.setEnabled(false);
    }
  }



}

class MainWindow_this_windowAdapter extends java.awt.event.WindowAdapter {
  MainWindow adaptee;

  MainWindow_this_windowAdapter(MainWindow adaptee) {
    this.adaptee = adaptee;
  }
  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class MainWindow_windows_menuAdapter implements javax.swing.event.MenuListener {
  MainWindow adaptee;

  MainWindow_windows_menuAdapter(MainWindow adaptee) {
    this.adaptee = adaptee;
  }
  public void menuCanceled(MenuEvent e) {
  }
  public void menuDeselected(MenuEvent e) {
  }
  public void menuSelected(MenuEvent e) {
    adaptee.windows_menuSelected(e);
  }
}

class MainWindow_modelmenu_menuAdapter implements javax.swing.event.MenuListener {
  MainWindow adaptee;

  MainWindow_modelmenu_menuAdapter(MainWindow adaptee) {
    this.adaptee = adaptee;
  }
  public void menuCanceled(MenuEvent e) {
  }
  public void menuDeselected(MenuEvent e) {
  }
  public void menuSelected(MenuEvent e) {
    adaptee.modelmenu_menuSelected(e);
  }
}

class FRMFileFilter extends javax.swing.filechooser.FileFilter {
  public boolean accept(File f) {
    return f.isDirectory() ||  f.getName().endsWith(".frm");
  }
  public String getDescription() {
    return "MRM Formal Description";
  }
}

class RRMFileFilter extends javax.swing.filechooser.FileFilter {
  public boolean accept(File f) {
    return f.isDirectory() || f.getName().endsWith(".rrm");
  }
  public String getDescription() {
    return "MRM Raw Description";
  }
}

class MRWFileFilter extends javax.swing.filechooser.FileFilter {
  public boolean accept(File f) {
    return f.isDirectory() || f.getName().endsWith(".mws");
  }
  public String getDescription() {
    return "MRMSolve Workspace File";
  }
}

class MainWindow_analyse_menuAdapter implements javax.swing.event.MenuListener {
  MainWindow adaptee;

  MainWindow_analyse_menuAdapter(MainWindow adaptee) {
    this.adaptee = adaptee;
  }
  public void menuCanceled(MenuEvent e) {
  }
  public void menuDeselected(MenuEvent e) {
  }
  public void menuSelected(MenuEvent e) {
    adaptee.analyse_menuSelected(e);
  }
}

class MainWindow_file_menuAdapter implements javax.swing.event.MenuListener {
  MainWindow adaptee;

  MainWindow_file_menuAdapter(MainWindow adaptee) {
    this.adaptee = adaptee;
  }
  public void menuCanceled(MenuEvent e) {
  }
  public void menuDeselected(MenuEvent e) {
  }
  public void menuSelected(MenuEvent e) {
    adaptee.file_menuSelected(e);
  }
}

