import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class OptionPanel extends JPanel {

  Option[] options;
  JComponent[] components;
  String name;

  public OptionPanel(String name, Option[] options) {

    this.name = name;
    setLayout (new BorderLayout ());
    JPanel parent = this;

    this.options = options;
    components = new JComponent[options.length];
    for (int i=0; i<options.length; i++) {
      JPanel p = new JPanel (new BorderLayout ());
      Border b = BorderFactory.createEmptyBorder(8,4,8,4);
      p.setBorder(b);
      parent.add(p, BorderLayout.NORTH);
      JPanel p2 = new JPanel (new BorderLayout ());
      parent.add(p2, BorderLayout.CENTER);
      parent = p2;

      Option o = options[i];
      switch (o.getType()) {
        case Option.BOOL:
          JPanel xp = new JPanel (new BorderLayout (4, 0));
          xp.add(new JLabel (o.getName()+":"), BorderLayout.WEST);
          JCheckBox cb = new JCheckBox ("", ((Boolean)o.getValue()).booleanValue());
          components[i] = cb;
          xp.add(cb, BorderLayout.CENTER);
          xp.add(new JButton (new OptionHelpAction (o.getDescription())), BorderLayout.EAST);
          p.add(xp, BorderLayout.CENTER);
          break;
        case Option.DOUBLE:
          JPanel dp = new JPanel (new BorderLayout (4, 0));
          dp.add(new JLabel (o.getName()+":"), BorderLayout.WEST);
          JTextField tf = new JTextField (o.getValue().toString(),6);
          components[i] = tf;
          dp.add(tf, BorderLayout.CENTER);
          dp.add(new JButton (new OptionHelpAction (o.getDescription())), BorderLayout.EAST);
          p.add(dp, BorderLayout.CENTER);
          break;
        case Option.STRING:
          JPanel sp = new JPanel (new BorderLayout (4, 0));
          sp.add(new JLabel (o.getName()+":"), BorderLayout.WEST);
          JTextField sf = new JTextField (o.getValue().toString(),15);
          components[i] = sf;
          sp.add(sf, BorderLayout.CENTER);
          sp.add(new JButton (new OptionHelpAction (o.getDescription())), BorderLayout.EAST);
          p.add(sp, BorderLayout.CENTER);
          break;
        case Option.CHOICE:
          JPanel cp = new JPanel (new BorderLayout (4, 0));
          cp.add(new JLabel (o.getName()+":"), BorderLayout.WEST);
          JComboBox cbb = new JComboBox (o.getChoice());
          cbb.setSelectedIndex(((Integer)o.getValue()).intValue());
          components[i] = cbb;
          cp.add(cbb, BorderLayout.CENTER);
          cp.add(new JButton (new OptionHelpAction (o.getDescription())), BorderLayout.EAST);
          p.add(cp, BorderLayout.CENTER);
          break;
      }
    }
  }

  public void start () {
  }

  public void close () {
  }

  public void apply () {

    for (int i=0; i<options.length; i++) {
      switch (options[i].type) {
        case Option.BOOL:
          options[i].setValue(new Boolean(((JCheckBox)components[i]).isSelected()));
          break;
        case Option.STRING:
          options[i].setValue(new String(((JTextField)components[i]).getText()));
          break;
        case Option.DOUBLE:
          try {
            options[i].setValue(new Double(Double.parseDouble( ( (JTextField)
                components[i]).getText())));
          }
          catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(null, "Bad number format at '"+options[i].getName()+"'!", "Bad number format!", JOptionPane.ERROR_MESSAGE);
          }
          break;
        case Option.CHOICE:
          options[i].setValue(new Integer(((JComboBox)components[i]).getSelectedIndex()));
          break;
      }
    }
  }

  public void resetDefaults () {

    for (int i=0; i<options.length; i++) {
      switch (options[i].type) {
        case Option.BOOL:
          ((JCheckBox)components[i]).setSelected(((Boolean)options[i].getDefault()).booleanValue());
          break;
        case Option.STRING:
          ((JTextField)components[i]).setText((String)options[i].getDefault());
          break;
        case Option.DOUBLE:
          ((JTextField)components[i]).setText(((Double)options[i].getDefault()).toString());
          break;
        case Option.CHOICE:
          ((JComboBox)components[i]).setSelectedIndex(((Integer)options[i].getDefault()).intValue());
          break;
      }
    }
  }

  public void save (PrintWriter writer) {

    writer.println("["+name+"]");

    for (int i=0; i<options.length; i++) {
      writer.print (options[i].getName() + "=");
      switch (options[i].type) {
        case Option.BOOL:
          if (((JCheckBox)components[i]).isSelected())
            writer.println("yes;");
          else
            writer.println("no;");
          break;
        case Option.STRING:
          writer.println(((JTextField)components[i]).getText()+";");
          break;
        case Option.DOUBLE:
          writer.println(((JTextField)components[i]).getText()+";");
          break;
        case Option.CHOICE:
          writer.println(((JComboBox)components[i]).getSelectedIndex()+";");
          break;
      }
    }
    writer.println();
  }

  public void load (String optionFile) {

    BufferedReader reader = new BufferedReader (new StringReader (optionFile));

    try {
      String line = reader.readLine();
      int state = 0;
      int linenum = 1;
      while (line != null) {
        if (state==0 && line.startsWith("["+name+"]"))
          state = 1;
        else if (state==1 && line.startsWith("["))
          break;
        else if (state==1) {
          StringTokenizer stk = new StringTokenizer (line, "=;");
          if (stk.hasMoreTokens()) {
            try {
              String name = stk.nextToken();
              String value = stk.nextToken();
              int i = 0;
              for (i = 0; i < options.length; i++)
                if (options[i].getName().equals(name))
                  break;
              if (i == options.length)
                JOptionPane.showMessageDialog(null,
                                              "Option not found (" + name + ")!\n",
                                              "Option file error!",
                                              JOptionPane.ERROR_MESSAGE);
              else {
                switch (options[i].type) {
                  case Option.BOOL:
                    if (value.equals("yes"))
                      ( (JCheckBox) components[i]).setSelected(true);
                    else
                      ( (JCheckBox) components[i]).setSelected(false);
                    break;
                  case Option.STRING:
                    ( (JTextField) components[i]).setText(value);
                    break;
                  case Option.DOUBLE:
                    ( (JTextField) components[i]).setText(value);
                    break;
                  case Option.CHOICE:
                    ( (JComboBox) components[i]).setSelectedIndex(Integer.
                        parseInt(value));
                    break;
                }
              }
            }
            catch (NoSuchElementException ex) {
              JOptionPane.showMessageDialog(null,
                                            "Bad option file format (line " +
                                            linenum + ") in section ["+name+"]!\n" + ex.toString(),
                                            "Option file error!",
                                            JOptionPane.ERROR_MESSAGE);
            }
          }
        }
        line = reader.readLine();
        linenum++;
      }
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(null, "Bad option file format!\n"+ex.toString(),"Option file error!", JOptionPane.ERROR_MESSAGE);
    }
    apply ();
  }
}

class OptionHelpAction implements Action {

  public static Icon hi = new ImageIcon ("images/help16.gif");
  String help = null;

  public OptionHelpAction (String help) {
    this.help = help;
  }

  public void addPropertyChangeListener (PropertyChangeListener listener) {
  }

  public Object getValue(String key) {
    if (key.equals(Action.NAME)) {
      return null;
    }
    else if (key.equals(Action.SMALL_ICON)) {
      return hi;
    }
    else if (key.equals("Help")) {
      return help;
    }
    return null;
  }

  public boolean isEnabled() {
    return help!=null;
  }

  public void putValue (String key, Object value) {
  }

  public void removePropertyChangeListener(PropertyChangeListener listener) {
  }

  public void setEnabled(boolean b) {
  }

  public void actionPerformed (ActionEvent ae) {
    HTMLDialog htmld = new HTMLDialog ("Description", 300, 250);
    htmld.showString (ColorChanger.changeColor(help));
  }
}
