
import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;

/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class OptionsDialog extends JDialog implements TreeSelectionListener {
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JButton ok = new JButton();
  JButton cancel = new JButton();
  JPanel jPanel2 = new JPanel();
  JSplitPane jSplitPane1 = new JSplitPane();
  BorderLayout borderLayout2 = new BorderLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JTree tree = new JTree();
  JScrollPane optionScroll = new JScrollPane();
  JPanel optionPanel = new JPanel();
  MainWindow mainw;
  boolean globalselected = false;
  OptionPanel globalPanel;
  Hashtable mrmTypes = new Hashtable();
  OptionPanel current = null;
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel jPanel3 = new JPanel();
  JPanel jPanel4 = new JPanel();
  JButton defaults = new JButton();
  JButton save = new JButton();
  JButton load = new JButton();

  public OptionsDialog(MainWindow mw) throws HeadlessException {
    mainw = mw;
    Option[] opts = new Option[mainw.options.size()];
    Enumeration en = mainw.options.elements();
    int i = 0;
    while (en.hasMoreElements())
      opts[i++] = (Option) en.nextElement();
    globalPanel = new OptionPanel("Global Options", opts);
    try {
      jbInit();
      setSize(600, 300);
      Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mine = getSize();
      setLocation( (screen.width - mine.width) / 2,
                  (screen.height - mine.height) / 2);
      fillOptionTree();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void fillOptionTree() {
    DefaultMutableTreeNode top = new DefaultMutableTreeNode("Preferences");
    DefaultTreeModel dtm = new DefaultTreeModel(top);
    top.add(new DefaultMutableTreeNode("Global Options"));
    String so = "MRM Solver Options";
    DefaultMutableTreeNode dmts = new DefaultMutableTreeNode(so);
    top.add(dmts);
    Iterator i = mainw.mrmTypes.iterator();
    while (i.hasNext()) {
      MRMType mt = (MRMType) i.next();
      String s = mt.getName();
      OptionPanel op = new OptionPanel(s, mt.getOptions());
      mrmTypes.put(s, op);
      dmts.add(new DefaultMutableTreeNode(s));
    }
    tree.setModel(dtm);
    tree.addTreeSelectionListener(this);
  }

  public void switchPanel(OptionPanel op) {

    if (current != null && current != op) {
      current.close();
      optionPanel.removeAll();
    }
    if (op != null) {
      current = op;
      current.start();
      optionPanel.add(current);
    }
    optionPanel.invalidate();
    optionScroll.getViewport().revalidate();
    optionScroll.getViewport().repaint();
  }

  public void valueChanged(TreeSelectionEvent e) {
    TreePath tp = tree.getSelectionPath();
    if (tp != null) {
      String s = tp.getLastPathComponent().toString();
      if (s.equals("Global Options")) {
        switchPanel(globalPanel);
      }
      else {
        Object o = mrmTypes.get(s);
        if (o != null && o instanceof OptionPanel) {
          switchPanel( (OptionPanel) o);
        }
        else
          switchPanel(null);
      }
    }
  }

  private void jbInit() throws Exception {
    this.setModal(true);
    this.setTitle("Options");
    this.getContentPane().setLayout(borderLayout1);
    ok.setText("OK");
    ok.addActionListener(new OptionsDialog_ok_actionAdapter(this));
    cancel.setText("Cancel");
    cancel.addActionListener(new OptionsDialog_cancel_actionAdapter(this));
    jPanel2.setLayout(borderLayout2);
    jSplitPane1.setLastDividerLocation(220);
    jPanel1.setLayout(borderLayout3);
    defaults.setText("Defaults");
    defaults.addActionListener(new OptionsDialog_defaults_actionAdapter(this));
    save.setText("Save");
    save.addActionListener(new OptionsDialog_save_actionAdapter(this));
    load.setText("Load");
    load.addActionListener(new OptionsDialog_load_actionAdapter(this));
    this.getContentPane().add(jPanel1, BorderLayout.SOUTH);
    jPanel1.add(jPanel3, BorderLayout.EAST);
    jPanel3.add(ok, null);
    jPanel3.add(cancel, null);
    jPanel1.add(jPanel4, BorderLayout.WEST);
    jPanel4.add(load, null);
    jPanel4.add(save, null);
    jPanel4.add(defaults, null);
    this.getContentPane().add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(jSplitPane1, BorderLayout.CENTER);
    jSplitPane1.add(jScrollPane1, JSplitPane.LEFT);
    jSplitPane1.add(optionScroll, JSplitPane.RIGHT);
    optionScroll.getViewport().add(optionPanel, null);
    jScrollPane1.getViewport().add(tree, null);
    jSplitPane1.setDividerLocation(220);
  }

  void ok_actionPerformed(ActionEvent e) {
    globalPanel.apply();
    Enumeration en = mrmTypes.elements();
    while (en.hasMoreElements())
      ( (OptionPanel) en.nextElement()).apply();
    setVisible(false);
  }

  void updateUI () {
    SwingUtilities.updateComponentTreeUI(this);
    SwingUtilities.updateComponentTreeUI(globalPanel);
    Enumeration en = mrmTypes.elements();
    while (en.hasMoreElements())
      SwingUtilities.updateComponentTreeUI((OptionPanel) en.nextElement());
  }

  void cancel_actionPerformed(ActionEvent e) {
    setVisible(false);
  }

  void defaults_actionPerformed(ActionEvent e) {
    int retVal = JOptionPane.showConfirmDialog(this,
        "Restore default options? Current settings will be lost!",
                                               "Are you sure?",
                                               JOptionPane.YES_NO_CANCEL_OPTION);
    if (retVal == JOptionPane.YES_OPTION) {
      globalPanel.resetDefaults();
      Enumeration en = mrmTypes.elements();
      while (en.hasMoreElements())
        ( (OptionPanel) en.nextElement()).resetDefaults();
      optionPanel.invalidate();
      optionScroll.getViewport().revalidate();
      optionScroll.getViewport().repaint();
    }
  }

  void save_actionPerformed(ActionEvent e) {
    globalPanel.apply();
    Enumeration en = mrmTypes.elements();
    while (en.hasMoreElements())
      ( (OptionPanel) en.nextElement()).apply();
    try {
      PrintWriter writer = new PrintWriter(new FileWriter("options"));
      globalPanel.save(writer);
      en = mrmTypes.elements();
      while (en.hasMoreElements())
        ( (OptionPanel) en.nextElement()).save(writer);
      writer.close();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,
                                    "Error writing options file!\n" + ex.toString(),
                                    "Error!", JOptionPane.ERROR_MESSAGE);
    }

  }

  void loadOptions() throws IOException {

    BufferedReader reader = new BufferedReader(new FileReader("options"));
    String options = "";
    String line = reader.readLine();
    while (line != null) {
      options += line + "\n";
      line = reader.readLine();
    }
    reader.close();
    globalPanel.load(options);
    Enumeration en = mrmTypes.elements();
    while (en.hasMoreElements())
      ( (OptionPanel) en.nextElement()).load(options);
  }

  void load_actionPerformed(ActionEvent e) {
    try {
      loadOptions();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,
                                    "Error loading options file!\n" +
                                    ex.toString(),
                                    "Error!", JOptionPane.ERROR_MESSAGE);
    }
  }
}

class OptionsDialog_ok_actionAdapter implements java.awt.event.ActionListener {
  OptionsDialog adaptee;

  OptionsDialog_ok_actionAdapter(OptionsDialog adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.ok_actionPerformed(e);
  }
}

class OptionsDialog_cancel_actionAdapter implements java.awt.event.ActionListener {
  OptionsDialog adaptee;

  OptionsDialog_cancel_actionAdapter(OptionsDialog adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.cancel_actionPerformed(e);
  }
}

class OptionsDialog_defaults_actionAdapter implements java.awt.event.ActionListener {
  OptionsDialog adaptee;

  OptionsDialog_defaults_actionAdapter(OptionsDialog adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.defaults_actionPerformed(e);
  }
}

class OptionsDialog_save_actionAdapter implements java.awt.event.ActionListener {
  OptionsDialog adaptee;

  OptionsDialog_save_actionAdapter(OptionsDialog adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.save_actionPerformed(e);
  }
}

class OptionsDialog_load_actionAdapter implements java.awt.event.ActionListener {
  OptionsDialog adaptee;

  OptionsDialog_load_actionAdapter(OptionsDialog adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.load_actionPerformed(e);
  }
}


