import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;

public class PRSMRM extends MRM {


  public PRSMRM (MRMType t, String shname, String fname, boolean isRaw) {
    super (t, shname, fname, isRaw);

    if (isRaw) {
      addMatrix (new UMatrix ("Q","Rate"),"Generator of the Markov chain");
      addVector (new UVector ("R","Reward rate"),"Reward rates");
      addVector (new PUVector ("P0","Probability"),"Initial probability vector");
    }
    else {
      String sablon = "type prs;\n\nConst\n\t//enumerate constants here\nVar\n\t//variable definitions\nState\n\nMatrix Q\n\t//define generator of the structure state process\nVector R\n\t//define reward rate vector\nVector P0\n\t//define initial probability vector\n";
      modelDef.setText(sablon);
      setDirty (false);
    }
  }

  public boolean extractDescription (String qfname, String rfname, String ifname, boolean generatep0, MRMParameterSet ps) { // return wether p0 was specified

    boolean hasp0 = false;
    boolean embedded = false;
    try {
      BufferedReader reader = new BufferedReader (new StringReader (modelDef.getText()));
      PrintStream qfile = new PrintStream (new FileOutputStream(qfname));
      PrintStream rfile = new PrintStream (new FileOutputStream(rfname));
      PrintStream ifile = new PrintStream (new FileOutputStream(ifname));
      int state = 0; // =0: blind copy; =1: inside const; =2: inside q; =3: inside r; =4: inside p0
      String delim = new String (" \n\r\t,;:(){}[]-><=");
      String line = reader.readLine();
      while (line != null) {
        StringTokenizer tk = new StringTokenizer(line, delim);
        try {
          String first = tk.nextToken();
          switch (state) {
            case 0:
              if (first.equalsIgnoreCase("type")) {
                line = reader.readLine();
                continue;
              }
              else if (first.equals("Const")) {
                if (ps==null) {
                  qfile.println(line);
                  rfile.println(line);
                  ifile.println(line);
                }
                else {
                  Iterator i = ps.parameters.iterator();
                  qfile.println("Const");
                  rfile.println("Const");
                  ifile.println("Const");
                  while (i.hasNext()) {
                    Parameter p = (Parameter)i.next();
                    qfile.println("\t"+p.name+" = "+p.value+";");
                    rfile.println("\t"+p.name+" = "+p.value+";");
                    ifile.println("\t"+p.name+" = "+p.value+";");
                  }
                  qfile.println();
                  rfile.println();
                  ifile.println();
                }
                state = 1;
              }
              else if (first.equals("Matrix")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("Q")) {
                    qfile.println("Matrix");
                    state = 2;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal matrix id: "+next,
                                                  "Illegal matrix id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No matrix name found",
                                                "No matrix name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else if (first.equals("Vector")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("R")) {
                    rfile.println("Vector");
                    state = 3;
                  }
                  else if (next.equals("P0")) {
                    ifile.println("Vector");
                    state = 4;
                    hasp0 = true;
                    if (generatep0 && tk.hasMoreTokens() && tk.nextToken().equals("embedded"))
                      embedded = true;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                  "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else {
                qfile.println(line);
                rfile.println(line);
                ifile.println(line);
              }
              break;
            case 1:
              if (first.equals("Var") || first.equals("State")) {
                qfile.println(line);
                rfile.println(line);
                ifile.println(line);
                state = 0;
              }
              else if (first.equals("Matrix")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("Q")) {
                    qfile.println("Matrix");
                    state = 2;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal matrix id: "+next,
                                                  "Illegal matrix id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No matrix name found",
                                                "No matrix name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else if (first.equals("Vector")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("R")) {
                    rfile.println("Vector");
                    state = 3;
                  }
                  else if (next.equals("P0")) {
                    ifile.println("Vector");
                    state = 4;
                    hasp0 = true;
                    if (generatep0 && tk.hasMoreTokens() && tk.nextToken().equals("embedded"))
                      embedded = true;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                  "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else {
                if (ps==null) {
                  qfile.println(line);
                  rfile.println(line);
                  ifile.println(line);
                }
              }
              break;
            case 2:
              if (first.equals("Vector")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("R")) {
                    rfile.println("Vector");
                    state = 3;
                  }
                  else if (next.equals("P0")) {
                    ifile.println("Vector");
                    state = 4;
                    hasp0 = true;
                    if (generatep0 && tk.hasMoreTokens() && tk.nextToken().equals("embedded"))
                      embedded = true;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                  "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else
                qfile.println(line);
              break;
            case 3:
              if (first.equals("Vector")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("P0")) {
                    ifile.println("Vector");
                    state = 4;
                    hasp0 = true;
                    if (generatep0 && tk.hasMoreTokens() && tk.nextToken().equals("embedded"))
                      embedded = true;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                  "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else
                rfile.println(line);
              break;
            case 4:
              if (embedded) {
                tk = new StringTokenizer(line, delim);
                first = tk.nextToken();
                PrintStream p0qfile = new PrintStream(new FileOutputStream(
                    tmpdir + "/p0q.m"));
                PrintStream p0mfile = new PrintStream(new FileOutputStream(
                    tmpdir + "/p0m.map"));
                // copy description of the embedded matkov chain to an other file
                while (line != null &&
                       (first == null ||
                        (first != null && !first.equals("Vector") &&
                         !first.equals("Matrix") && !first.equals("Mapping")))) {
                  if (first != null && first.equals("P0generator"))
                    p0qfile.println("Matrix");
                  else
                    p0qfile.println(line);
                  line = reader.readLine();
                  if (line != null) {
                    try {
                      tk = new StringTokenizer(line, delim);
                      first = tk.nextToken();
                    }
                    catch (NoSuchElementException ex) {
                      first = null;
                    }
                  }
                }
                // copy mapping to an other file
                line = reader.readLine();
                while (line != null &&
                       (first == null ||
                        (first != null && !first.equals("Vector") &&
                         !first.equals("Matrix")))) {
                  p0mfile.println(line);
                  line = reader.readLine();
                  if (line != null) {
                    try {
                      tk = new StringTokenizer(line, delim);
                      first = tk.nextToken();
                    }
                    catch (NoSuchElementException ex) {
                      first = null;
                    }
                  }
                }
                // execute tool to compute steady state probability vector
                p0qfile.close();
                p0mfile.close();
                JDialog waitdlg = new JDialog ();
                waitdlg.setModal(false);
                waitdlg.setSize(350, 100);
                Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
                Dimension mine = getSize();
                waitdlg.setLocation( (screen.width - mine.width) / 2,
                             (screen.height - mine.height) / 2);
                waitdlg.getContentPane().add(new Label ("Computing steady state of the embedded model..."));
                waitdlg.setVisible(true);
                String commandline = "bin/steadystate " + tmpdir + "/p0q.m " + tmpdir + "/p0q.steadystate";
                System.out.println(commandline);
                CommandRunner cmd = new CommandRunner(commandline);
                cmd.start();
                while (!cmd.running) {}
                System.out.println ("Fut...");
                while (cmd.running) {}
                commandline = "bin/compilerule " + tmpdir + "/q.m " + tmpdir + "/q.compiled";
                System.out.println(commandline);
                cmd = new CommandRunner(commandline);
                cmd.start();
                while (!cmd.running) {}
                System.out.println ("Fut...");
                while (cmd.running) {}
                commandline = "bin/compilerule " + tmpdir + "/p0q.m " + tmpdir + "/p0q.compiled";
                System.out.println(commandline);
                cmd = new CommandRunner(commandline);
                cmd.start();
                while (!cmd.running) {}
                System.out.println ("Fut...");
                while (cmd.running) {}
                commandline = "bin/mapping " + tmpdir + "/p0m.map " +tmpdir + "/p0q.compiled " +tmpdir + "/q.compiled " + tmpdir + "/p0q.steadystate " + tmpdir + "/p0q.probs";
                System.out.println(commandline);
                cmd = new CommandRunner(commandline);
                cmd.start();
                while (!cmd.running) {}
                System.out.println ("Fut...");
                while (cmd.running) {}
                BufferedReader p0reader = new BufferedReader (new InputStreamReader (new FileInputStream(tmpdir+"/p0q.probs")));
                String p0l = p0reader.readLine();
                while (p0l!=null) {
                  ifile.println(p0l);
                  p0l = p0reader.readLine();
                }
                p0reader.close();
                 waitdlg.setVisible(false);
              }
              else
                ifile.println(line);
          }
        }
        catch (NoSuchElementException ex) {
//          file.println(line);
        }
        line = reader.readLine();
      }
      qfile.close();
      rfile.close();
      ifile.close();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error writing tmp files", JOptionPane.ERROR_MESSAGE);
    }
    return hasp0;
  }

  public MRMParameterSet getParameterSet () {
    MRMParameterSet ps = new MRMParameterSet ();
    if (isRaw)
      return ps;
    else {
      extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",false,null);
      Parameter p;
      try {
        Process process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir +
            "/q.m " + tmpdir + "/q.compiled");
        System.out.println("bin/compilerule " + tmpdir + "/q.m " + tmpdir +
                           "/q.compiled");
        int exitVal = process.waitFor();
        if (exitVal > 0)
          return ps;
        try {
          BufferedReader reader = new BufferedReader (new FileReader(tmpdir+"/q.compiled"));
          String delim = new String (" \n\r\t,;:(){}[]><=");
          String line = reader.readLine();
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, delim);
            String first = tk.nextToken();
            if (!first.equals("Const"))
              break;
            String id = tk.nextToken();
            String vals = tk.nextToken();
            double val = Double.parseDouble(vals);
            p = new Parameter (id, val);
            ps.parameters.add(p);
            line = reader.readLine();
          }
          reader.close();
        }
        catch (IOException e) {
          JOptionPane.showMessageDialog(this, e.toString(),"Tmp file 'q.compiled' not found!", JOptionPane.ERROR_MESSAGE);
        }
        catch (NoSuchElementException x) {
          JOptionPane.showMessageDialog(this, x.toString(),"Tmp file 'q.compiled' has bad format!", JOptionPane.ERROR_MESSAGE);
        }
      }
      catch (IOException ex) {
        JOptionPane.showMessageDialog(this, ex.toString(),"compilefrmq not found!", JOptionPane.ERROR_MESSAGE);
      }
      catch (InterruptedException ex2) {
        JOptionPane.showMessageDialog(this, ex2.toString(),"compilefrmq interrupted!", JOptionPane.ERROR_MESSAGE);
      }
      return ps;
    }

  }

  public void getARMoments (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (running.equalsIgnoreCase("Time")) {
      MRMParameterSet mp = new MRMParameterSet (p);
      mp.delParameter("Time");
      runAnalysis(true, mp, running, start, end, steps, numOfMoms, normalized);
    }
    else
      runPAnalysis (true, p, running, start, end, steps, numOfMoms, normalized);
    p.delParameter(running);
  }

  public void getCTMoments (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (running.equalsIgnoreCase("Reward")) {
      MRMParameterSet mp = new MRMParameterSet (p);
      mp.delParameter("Reward");
      runAnalysis(false, mp, running, start, end, steps, numOfMoms, normalized);
    }
    else
      runPAnalysis (false, p, running, start, end, steps, numOfMoms, normalized);
    p.delParameter(running);
  }

  public String checkModel () {
    if (isRaw)
      return checkRaw ();
    else {
      // check type
      String mnemonic = null;
      try {
        BufferedReader file = new BufferedReader(new StringReader(modelDef.getText()));
        String delim = new String (" \n\r\t,;:(){}[]-><=");
        String line = file.readLine();
        while (line != null) {
          StringTokenizer tk = new StringTokenizer (line, delim);
          if (tk.hasMoreTokens()) {
            if (tk.nextToken().equals("type")) {
              mnemonic = tk.nextToken();
            }
            break;
          }
          line = file.readLine();
        }
      }
      catch (IOException ex) {}
      if (mnemonic==null) {
        return "Type specification missing,\nplease insert 'type = prs;' line at the beginning of the file!";
      }
      else if (!mnemonic.equalsIgnoreCase(type.getMnemonic())) {
        return "Bad type specification ("+mnemonic+"),\nplease insert 'type = prs;' line at the beginning of the file!\n(type cannot be changed after openeing the file)";
      }

      // check remaining description
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v", tmpdir + "/p0.v", true,null);
      Parameter p;
      try {
        Process process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/q.m " + tmpdir + "/q.compiled");
        int iq = process.waitFor();
        process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/r.v " + tmpdir + "/r.compiled");
        int ir = process.waitFor();
        int ip0 = 0;
        if (hasp0) {
          process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/p0.v " + tmpdir + "/p0.compiled");
          ip0 = process.waitFor();
        }
        System.out.println("Q: "+iq+", R: "+ir+", P0: "+ip0);
        String error = null;
        if (iq!=0) {
          error = "Error in Q matrix definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/q.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        else if (ir!=0) {
          error = "Error in R vector definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/r.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        else if (hasp0 && ip0!=0) {
          error = "Error in p0 vector definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/p0.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        return error;
      }
      catch (InterruptedException ex) {
      }
      catch (IOException ex) {

      }
    }
    return null;
  }

  public void runAnalysis (boolean ar, MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (isRaw) {
      analyserProcess = null;
      JOptionPane.showMessageDialog(this, "Raw input not yet supported in prs moment analysis!", "Can not start analysis", JOptionPane.ERROR_MESSAGE);
    }
    else {
      double[][] result = new double[steps][];
      for (int i=0; i<steps; i++)
        result[i] = new double [numOfMoms+3];

      boolean hasp0 = extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",true, p);
      String p0 = "";
      if (hasp0)
        p0 = tmpdir+"/p0.v ";
      String norm = "NORMAL";
      if (normalized)
        norm = "NORMALIZED";
      String commandline;
      if (ar)
        commandline = "bin/mrmsolve "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt "+numOfMoms+" AR "+norm+" "+start+" "+end+" "+steps;
      else
        commandline = "bin/mrmsolve "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt "+numOfMoms+" CT "+norm+" "+start+" "+end+" "+steps;

        System.out.println(commandline);
        CommandRunner cmd = new CommandRunner (commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start ();
        String msg = "";
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);

            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            int G = 0;
            String overall = "";
            String subtask = "";
            int overalltotal = 4;
            int overallcompleted = -1;
            int subtasktotal = 0;
            int subtaskcompleted = 0;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                  subtasktotal = states;
                }
                else if (line.startsWith("  Generating elements...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/4";
                  subtask = "Generating elements";
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/4";
                  subtask = "Performing Markov reward model analysis";
                  subtasktotal = 1;
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  subtaskcompleted = st;
                }
                else if (line.startsWith(" G=")) {
                  G = Integer.parseInt(line.substring(3));
                  subtasktotal = G;
                }
                else if (line.startsWith("  g=")) {
                  int g = Integer.parseInt(line.substring(4));
                  subtaskcompleted = g;
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
          }
          catch (InterruptedException ex2) {
            return;
          }
        }
        listener.updateStatus ("Overall progress: 4/4",4,4,"Analysis ready",1,1,msg);
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          int linenum = 0;
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            for (int i=0; i<numOfMoms+3; i++) {
              result[linenum][i] = Double.parseDouble(tk.nextToken());
            }
            linenum++;
            line = resreader.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
        listener.addResults(result);
        listener.finish ();
        analyserProcess = null;
    }
  }

  public void runPAnalysis (boolean ar, MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (isRaw) {
      analyserProcess = null;
      JOptionPane.showMessageDialog(this, "Raw input not yet supported in prs moment analysis!", "Can not start analysis", JOptionPane.ERROR_MESSAGE);
    }
    else {
      double[][] result = new double[1][];
      result[0] = new double [numOfMoms+3];
      for (int j=0; j<numOfMoms+3; j++)
        result[0][j] = 0.0;

      String norm = "NORMAL";
      if (normalized)
        norm = "NORMALIZED";
      double nvar;
      if (ar)
        nvar = p.getParameter("Time").value;
      else
        nvar = p.getParameter("Reward").value;
      String commandline;
      boolean hasp0 = extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",true, null);
      String p0 = "";
      if (hasp0)
        p0 = tmpdir+"/p0.v ";
      if (ar)
        commandline = "bin/mrmsolve.exe "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt " + numOfMoms + " AR " +
            norm + " " + nvar + " " + nvar + " 1";
      else
        commandline = "bin/mrmsolve.exe "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt " + numOfMoms + " CT " +
            norm + " " + nvar + " " + nvar + " 1";
      System.out.println(commandline);

      for (int s=0; s<steps; s++) {
        double cval = start + s * (end-start) / (steps-1);

        MRMParameterSet cps = new MRMParameterSet (p);
        if (ar)
          cps.delParameter("Time");
        else
          cps.delParameter("Reward");

        cps.getParameter(running).value = cval;
        double sperc = (double)(s+1) / steps;
        String overall = running+" = "+cval+" ("+(s+1)+"/"+steps+")";

        extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",true,cps);

        CommandRunner cmd = new CommandRunner(commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start();
        String msg = "";
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);
            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            double perc = 0;
            String subtask = "";
            int stage = 0;
            int G = 1;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                }
                else if (line.startsWith("  Generating elements...")) {
                  stage++;
                  subtask = "Generating elements";
                  perc = (stage - 1) / 4.0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  stage++;
                  subtask = "Performing Markov reward model analysis";
                  perc = (stage - 1) / 4.0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  perc = (stage - 1 + (double)st / states) / 4.0;
                }
                else if (line.startsWith(" G=")) {
                  G = Integer.parseInt(line.substring(3));
                }
                else if (line.startsWith("  g=")) {
                  int g = Integer.parseInt(line.substring(4));
                  perc = (stage - 1 + (double) g / G) / 4.0;
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus(overall, steps, s, subtask, 1000, (int)(perc*1000), msg);
          }
          catch (InterruptedException ex2) {
            return;
          }
        }
        listener.updateStatus (overall, steps, s, "Analysis ready", 1, 1, msg);
        if (cmd.aborted)
          break;
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          if (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            for (int i=0; i<numOfMoms+3; i++)
              result[0][i] = Double.parseDouble(tk.nextToken());
          }
          result[0][0] = cval;
          listener.addResults(result);
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
      }
      listener.finish ();
      analyserProcess = null;
    }
  }

  public void getCTDistributionfm (MRMParameterSet p, double x, double start, double end, int steps, boolean normalized, int numOfMoms) {

    if (isRaw) {
      analyserProcess = null;
      JOptionPane.showMessageDialog(this, "Raw input not yet supported in prs moment analysis!", "Can not start analysis", JOptionPane.ERROR_MESSAGE);
    }
    else {
      double[][] result = new double[1][];
      for (int i=0; i<1; i++)
        result[i] = new double [5];

      boolean hasp0 = extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",true, p);
      String p0 = "";
      if (hasp0)
        p0 = tmpdir+"/p0.v ";
      String norm = "NORMAL";
      if (normalized)
        norm = "NORMALIZED";
      String commandline = "bin/mrmsolve "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt "+numOfMoms+" AR "+norm+" "+start+" "+end+" "+steps;

      System.out.println(commandline);
      CommandRunner cmd = new CommandRunner (commandline);
      cmd.start();
      while (!cmd.running) {}
      listener.start ();
      String msg = "";
      while (cmd.running) {
        try {
          analyserProcess = cmd;
          Thread.currentThread().sleep(500);

          if (cmd.errorGobbler != null)
            msg = cmd.errorGobbler.s;

          BufferedReader reader = new BufferedReader(new StringReader(msg));
          int G = 0;
          String overall = "";
          String subtask = "";
          int overalltotal = 4;
          int overallcompleted = -1;
          int subtasktotal = 0;
          int subtaskcompleted = 0;
          int states = 1;

          try {
            String line = reader.readLine();
            while (line != null) {
              if (line.startsWith("  #States=")) {
                states = Integer.parseInt(line.substring(10));
                subtasktotal = states;
              }
              else if (line.startsWith("  Generating elements...")) {
                overallcompleted++;
                overall = "Overall progress: "+(overallcompleted)+"/4";
                subtask = "Generating elements";
                subtaskcompleted = 0;
              }
              else if (line.startsWith("Starting Markov reward model analysis...")) {
                overallcompleted++;
                overall = "Overall progress: "+(overallcompleted)+"/4";
                subtask = "Performing Markov reward model analysis";
                subtasktotal = 1;
                subtaskcompleted = 0;
              }
              else if (line.startsWith("    #Processed states =")) {
                int st = Integer.parseInt(line.substring(24));
                subtaskcompleted = st;
              }
              else if (line.startsWith(" G=")) {
                G = Integer.parseInt(line.substring(3));
                subtasktotal = G;
              }
              else if (line.startsWith("  g=")) {
                int g = Integer.parseInt(line.substring(4));
                subtaskcompleted = g;
              }
              line = reader.readLine();
            }
          }
          catch (IOException ex) {
            ex.printStackTrace();
          }
          listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
        }
        catch (InterruptedException ex2) {
          return;
        }
      }
      listener.updateStatus ("Overall progress: 4/4",4,4,"Moment analysis ready, starting distr. estim.",1,1,msg);
      try {
        BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
        String line = resreader.readLine();
        int linenum = 0;
        String momEstimLine = "";
        while (line!=null) {
          StringTokenizer tk = new StringTokenizer (line, " \t\n");
          String first = tk.nextToken();
          result[0][0] = Double.parseDouble(first);
//          momEstimLine = first;
          momEstimLine = "";
          for (int i=0; i<numOfMoms; i++)
            momEstimLine = momEstimLine + " " + tk.nextToken();
          double lower = Double.parseDouble(tk.nextToken());
          double upper = Double.parseDouble(tk.nextToken());
          PrintStream infile = new PrintStream (new FileOutputStream(tmpdir + "/estdistr.in"));
          infile.println("" + x + " " +lower + " " + upper + " " + momEstimLine);
          infile.close();
          Process process = Runtime.getRuntime().exec("bin/distributionestimation "+tmpdir+"/estdistr.in "+tmpdir+"/estdistr.out");
          process.waitFor ();
          BufferedReader outfile = new BufferedReader (new FileReader (tmpdir+"/estdistr.out"));
          String oline = outfile.readLine();
          StringTokenizer stk = new StringTokenizer (oline, " \t\n");
          stk.nextToken();
          for (int i=1; i<5; i++)
            result[0][i] = 1 - Double.parseDouble(stk.nextToken());
          listener.addResults(result);
          linenum++;
          line = resreader.readLine();
        }
      }
      catch (IOException ex) {
        JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
      }
      catch (InterruptedException ex2) {
        JOptionPane.showMessageDialog(this, ex2.toString(),"Could not execute distribution estimation tool!", JOptionPane.ERROR_MESSAGE);
      }
      listener.updateStatus ("Overall progress: 4/4",4,4,"Analysis ready.",1,1,msg);
      listener.finish ();
      analyserProcess = null;
  }
  }


  public String getStateSpaceHTMLString () {

    extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",false,null);
    try {
      Process process = Runtime.getRuntime().exec("bin/statespacegeneration "+tmpdir+"/q.m "+tmpdir+"/states.txt");
      System.out.println("bin/compilefrmq "+tmpdir+"/q.m "+tmpdir+"/q.compiled");
      process.waitFor ();
      BufferedReader reader = new BufferedReader (new FileReader (tmpdir+"/states.txt"));

      StringBuffer stateSpace = new StringBuffer("<HTML> <BODY bgcolor=%BG%> <H3 align=center>State Space of "+shortname+"</H3><HR>");
      stateSpace.append("<TABLE border=0 width=100% cellspacing=0><TR><TD><TABLE width=100% border=0 cellspacing=0>");
      String color;
      boolean even = true;
      String line = reader.readLine();
      String evenPrefix = "<TR bgcolor=%TABLE1%><TD align=center>";
      String oddPrefix = "<TR bgcolor=%TABLE2%><TD align=center>";
      String postfix = "</TD></TR>";
      while (line!=null) {
        if (even)
          stateSpace.append(evenPrefix);
        else
          stateSpace.append(oddPrefix);
        stateSpace.append(line);
        stateSpace.append(postfix);
        line = reader.readLine();
        even = !even;
      }
      stateSpace.append("</TABLE></TR></TD></TABLE></BODY></HTML>");
      return stateSpace.toString();
    }
    catch (InterruptedException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error during execution of statespace generation!", JOptionPane.ERROR_MESSAGE);
    }
    catch (IOException ex2) {
      JOptionPane.showMessageDialog(this, ex2.toString(),"Could not open state space file (states.txt)!", JOptionPane.ERROR_MESSAGE);
    }

    return "";
  }

  public void showStateSpace () {

    File f = new File (tmpdir+"/states.gif");
    f.delete();

    String commandLine = "bin/drawstates ";
    if (!isRaw) {
      extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/p0.v",false,null);
      commandLine += "-formalizm "+tmpdir+"/q.m -o "+tmpdir+"/states.gif -t "+tmpdir+"/states.dot";
    }
    else {
      save(tmpdir + "/model.rrm");
      commandLine += "-raw " + tmpdir + "/model.rrm -o " + tmpdir + "/states.gif -t " + tmpdir + "/states.dot";
    }

    System.out.println(commandLine);

    DotRunner dr = new DotRunner (mw, commandLine, tmpdir+"/states.gif");
    dr.runDot();
  }


  public void getARDistribution (MRMParameterSet p, double t, double start, double end, int steps, boolean normalized) {

    double[][] result=null;
    if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==0)
      result = silva (p, t, start, end, steps, normalized);
    else if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==1)
      result = bruno (p, t, start, end, steps, normalized);
    else if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==2)
      result = donatiello (p, t, start, end, steps, normalized);

    if (result!=null)
      listener.addResults(result);
    listener.finish ();
    analyserProcess = null;

  }

  public void getCTDistribution (MRMParameterSet p, double w, double start, double end, int steps, boolean normalized) {


    double[][] result = new double[1][];
    result[0] = new double[3];

    for (int i=0; i<steps; i++) {
      double[][] aresult = null;
      double t = start + i * (end-start) / (steps-1);
      if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==0)
        aresult = silva (p, t, w, w, 1, normalized);
      else if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==1)
        aresult = bruno (p, t, w, w, 1, normalized);
      else if (((Integer)(getType().getOption("Algorithm for AR distribution").getValue())).intValue()==2)
        aresult = donatiello (p, t, w, w, 1, normalized);

      result[0][0] = t;
      result[0][1] = 1.0 - aresult[0][1];
      result[0][2] = 1.0 - aresult[0][2];
      listener.addResults(result);
    }
    listener.finish ();
    analyserProcess = null;
  }

  public double[][] silva (MRMParameterSet p, double t, double start, double end, int steps, boolean normalized) {
    double[][] result = new double[steps][];
    for (int i=0; i<steps; i++)
      result[i] = new double [3];

    String commandline = "";
    analyserProcess = null;
    String norm = "";
    if (normalized)
      norm = "-normalized ";

    if (isRaw) {
      save (tmpdir+"/model.rrm");
      commandline = "bin/silva -raw " + tmpdir + "/model.rrm " +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }
    else {
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v",
                                         tmpdir + "/p0.v", true, p);
      String p0 = "";
      if (hasp0)
        p0 = "P0=" + tmpdir + "/p0.v ";

      commandline = "bin/silva -formalizm Q=" + tmpdir + "/q.m R=" +
          tmpdir + "/r.v " + p0 +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }

        System.out.println(commandline);
        CommandRunner cmd = new CommandRunner (commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start ();
        String msg = "";
        int overalltotal = 4;
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);

            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            String overall = "";
            String subtask = "";
            int overallcompleted = -1;
            int subtasktotal = 0;
            int subtaskcompleted = 0;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                  subtasktotal = states;
                }
                else if (line.startsWith("  Generating elements...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Generating elements";
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Performing Markov reward model analysis";
                  subtasktotal = steps;
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  subtaskcompleted = st;
                }
                else if (line.startsWith("  point")) {
                  StringTokenizer stk = new StringTokenizer (line.substring(8), " \t");
                  if (stk.hasMoreTokens()) {
                    int pt = Integer.parseInt(stk.nextToken());
                    subtaskcompleted = pt;
                  }
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
          }
          catch (InterruptedException ex2) {}
        }
        listener.updateStatus ("Overall progress: "+overalltotal+"/"+overalltotal,overalltotal,overalltotal,"Analysis ready",1,1,msg);
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          int linenum = 0;
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            result[linenum][0] = Double.parseDouble(tk.nextToken());
            result[linenum][1] = Double.parseDouble(tk.nextToken());
            result[linenum][2] = 1-result[linenum][1];
            linenum++;
            line = resreader.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
        return result;
  }

  public double[][] donatiello (MRMParameterSet p, double t, double start, double end, int steps, boolean normalized) {
    double[][] result = new double[steps][];
    for (int i=0; i<steps; i++)
      result[i] = new double [3];

    String commandline = "";
    analyserProcess = null;
    String norm = "";
    if (normalized)
      norm = "-normalized ";

    if (isRaw) {
      save (tmpdir+"/model.rrm");
      commandline = "bin/donatiello -raw " + tmpdir + "/model.rrm " +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }
    else {
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v",
                                         tmpdir + "/p0.v", true, p);
      String p0 = "";
      if (hasp0)
        p0 = "P0=" + tmpdir + "/p0.v ";

      commandline = "bin/donatiello -formalizm Q=" + tmpdir + "/q.m R=" +
          tmpdir + "/r.v " + p0 +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }

        System.out.println(commandline);
        CommandRunner cmd = new CommandRunner (commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start ();
        String msg = "";
        int overalltotal = 4;
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);

            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            String overall = "";
            String subtask = "";
            int overallcompleted = -1;
            int subtasktotal = 0;
            int subtaskcompleted = 0;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                  subtasktotal = states;
                }
                else if (line.startsWith("  Generating elements...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Generating elements";
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Performing Markov reward model analysis";
                  subtasktotal = steps;
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  subtaskcompleted = st;
                }
                else if (line.startsWith("  point")) {
                  StringTokenizer stk = new StringTokenizer (line.substring(8), " \t");
                  if (stk.hasMoreTokens()) {
                    int pt = Integer.parseInt(stk.nextToken());
                    subtaskcompleted = pt;
                  }
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
          }
          catch (InterruptedException ex2) {}
        }
        listener.updateStatus ("Overall progress: "+overalltotal+"/"+overalltotal,overalltotal,overalltotal,"Analysis ready",1,1,msg);
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          int linenum = 0;
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            result[linenum][0] = Double.parseDouble(tk.nextToken());
            result[linenum][1] = Double.parseDouble(tk.nextToken());
            result[linenum][2] = 1-result[linenum][1];
            linenum++;
            line = resreader.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
        return result;
  }

  public double[][] bruno (MRMParameterSet p, double t, double start, double end, int steps, boolean normalized) {
    double[][] result = new double[steps][];
    for (int i=0; i<steps; i++)
      result[i] = new double [3];

    String commandline = "";
    analyserProcess = null;
    String norm = "";
    if (normalized)
      norm = "-normalized ";

    if (isRaw) {
      save (tmpdir+"/model.rrm");
      commandline = "bin/bruno -raw " + tmpdir + "/model.rrm " +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }
    else {
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v",
                                         tmpdir + "/p0.v", true, p);
      String p0 = "";
      if (hasp0)
        p0 = "P0=" + tmpdir + "/p0.v ";

      commandline = "bin/bruno -formalizm Q=" + tmpdir + "/q.m R=" +
          tmpdir + "/r.v " + p0 +
          "-o " + tmpdir + "/res.txt" + norm + " -time " + t +
          " -reward from=" + start + " to=" + end + " steps=" + steps;
    }

        System.out.println(commandline);
        CommandRunner cmd = new CommandRunner (commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start ();
        String msg = "";
        int overalltotal = 4;
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);

            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            String overall = "";
            String subtask = "";
            int overallcompleted = -1;
            int subtasktotal = 0;
            int subtaskcompleted = 0;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                  subtasktotal = states;
                }
                else if (line.startsWith("  Generating elements...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Generating elements";
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Performing Markov reward model analysis";
                  subtasktotal = 0;
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  subtaskcompleted = st;
                }
                else if (line.startsWith("N = ")) {
                  int N = Integer.parseInt(line.substring(4));
                  subtasktotal = N;
                }
                else if (line.startsWith("  n = ")) {
                  int n = Integer.parseInt(line.substring(6));
                  subtaskcompleted = n;
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
          }
          catch (InterruptedException ex2) {}
        }
        listener.updateStatus ("Overall progress: "+overalltotal+"/"+overalltotal,overalltotal,overalltotal,"Analysis ready",1,1,msg);
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          int linenum = 0;
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            result[linenum][0] = Double.parseDouble(tk.nextToken());
            result[linenum][1] = Double.parseDouble(tk.nextToken());
            result[linenum][2] = 1-result[linenum][1];
            linenum++;
            line = resreader.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
        return result;
  }

}

