
import java.util.*;
/**
 * <p>Title: MRMSolve2</p>
 * <p>Description: The Markov Reward Model Solver</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: BUTE Dept. of Telecommunications</p>
 * @author not attributable
 * @version 2.0
 */

public class PRSType implements MRMType {

  Vector options = new Vector ();
  public static final String ALGO_SILVA = "Gail-Silva";
  public static final String ALGO_BRUNO = "Nabli-Sericola";
  public static final String ALGO_DONATIELLO = "Donatiello-Grassi";

  public PRSType() {

    Option algo = new Option ("Algorithm for AR distribution", Option.CHOICE, new Integer (0));
    options.add(algo);
    String[] algos = new String [3];
    algos[0] = ALGO_SILVA;
    algos[1] = ALGO_BRUNO;
    algos[2] = ALGO_DONATIELLO;
    algo.addChoice(algos);
    algo.addDescription("<html><body bgcolor=%BG%><h2>Algorithm</h2>The algorithm used by the prs mrm solver to compute the distribution of the accumulated reward. The selected algorithm can be fine tuned by the options below.</body></html>");

    Option epsilon = new Option ("Epsilon", Option.DOUBLE, new Double (0.0000001));
    options.add(epsilon);
    epsilon.addDescription("<html><body bgcolor=%BG%><h2>Epsilon</h2> The maximal allowed error of the computation."+
                         "Larger value speeds up computation but decreases accuracy.</body></html>");
  }

  public Option[] getOptions () {
    Option[] opts = new Option[options.size()];
    Enumeration e = options.elements();
    int i = 0;
    while (e.hasMoreElements())
      opts[i++] = (Option)e.nextElement();
    return opts;
  }

  public Option getOption (String name) {
    Iterator i = options.iterator();
    while (i.hasNext()) {
      Option o = (Option)i.next();
      if (o.getName().equals(name))
        return o;
    }
    return null;
  }

  public MRM getInstance (String shname, String fname, boolean isRaw) {
    return new PRSMRM (this, shname, fname, isRaw);
  }

  public String getMnemonic () {
    return "prs";
  }

  public String getName () {
    return "Preemptive Resume MRM";
  }

  public String getDescription () {
    return "<html><body><h2>Preemptive Resume MRM</h2></body></html>";
  }

  public boolean canComputeARMoms () {
    return true;
  }

  public boolean canComputeCTMoms () {
    return true;
  }

  public boolean canComputeARDistr () {
    return true;
  }

  public boolean canComputeCTDistr () {
    return true;
  }

  public String toString () {
    return getName ();
  }

  public boolean canDrawStateSpace () {
    return true;
  }
}
