import java.io.*;
import java.util.*;
import javax.swing.*;

public class SOMRM extends MRM {

  class SPUVector extends UVector {

    public SPUVector (String n, String l) {
      super (n, l);
    }

    public String check () {

      Iterator i = val.iterator ();
      Iterator j = state.iterator ();
      while (i.hasNext()) {
        double d = ( (Double) (i.next())).doubleValue();
        int e = ( (Integer) (j.next())).intValue();
        if (d<0.0 || d>1.0)
          return name+": elements of this vector must be between (0,1)! (element "+e+")";
      }
      return null;
    }
  }

  public SOMRM (MRMType t, String shname, String fname, boolean isRaw) {
    super (t, shname, fname, isRaw);

    if (isRaw) {
      addMatrix (new UMatrix ("Q","Rate"),"Generator of the Markov chain");
      addVector (new UVector ("R","Reward rate"),"Reward rates");
      addVector (new UVector ("S","Reward variance"),"Reward variances");
      addVector (new PUVector ("P0","Probability"),"Initial probability vector");
    }
  }

  public boolean extractDescription (String qfname, String rfname, String sfname, String ifname, MRMParameterSet ps) { // return wether p0 was specified

    boolean hasp0 = false;
    try {
      BufferedReader reader = new BufferedReader (new StringReader (modelDef.getText()));
      PrintStream qfile = new PrintStream (new FileOutputStream(qfname));
      PrintStream rfile = new PrintStream (new FileOutputStream(rfname));
      PrintStream sfile = new PrintStream (new FileOutputStream(sfname));
      PrintStream ifile = new PrintStream (new FileOutputStream(ifname));
      PrintStream active = null;
      String delim = new String (" \n\r\t,;:(){}[]-_><=");
      String line = reader.readLine();
      while (line != null) {
        StringTokenizer tk = new StringTokenizer(line, delim);
        try {
          String first = tk.nextToken();
          if (active==null) {
              if (first.equalsIgnoreCase("type")) {
                line = reader.readLine();
                continue;
              }
              else if (first.equals("Const") && ps!=null) {
                  // read all the constants
                  boolean ready = false;
                  while (!ready) {
                    line = reader.readLine();
                    if (line!=null) {
                      StringTokenizer stk = new StringTokenizer (line, delim);
                      try {
                        String ft = stk.nextToken();
                        if (ft.equalsIgnoreCase("var") || ft.equalsIgnoreCase("state") || ft.equalsIgnoreCase("matrix") || ft.equalsIgnoreCase("vector"))
                          ready = true;
                      }
                      catch (NoSuchElementException e) {}
                    }
                    else {
                      ready = true;
                      return hasp0;
                    }
                  }
                  Iterator i = ps.parameters.iterator();
                  qfile.println("Const");
                  rfile.println("Const");
                  sfile.println("Const");
                  ifile.println("Const");
                  while (i.hasNext()) {
                    Parameter p = (Parameter)i.next();
                    qfile.println("\t"+p.name+" = "+p.value+";");
                    rfile.println("\t"+p.name+" = "+p.value+";");
                    sfile.println("\t"+p.name+" = "+p.value+";");
                    ifile.println("\t"+p.name+" = "+p.value+";");
                  }
                  qfile.println();
                  rfile.println();
                  sfile.println();
                  ifile.println();
                  continue;
                }
                else if (first.equals("Matrix")) {
                  if (tk.hasMoreTokens()) {
                    String next = tk.nextToken();
                    if (next.equals("Q")) {
                      qfile.println("Matrix");
                      active = qfile;
                    }
                    else {
                      JOptionPane.showMessageDialog(this, "Illegal matrix id: "+next,
                                                    "Illegal matrix id: "+next,
                                                    JOptionPane.ERROR_MESSAGE);
                    }
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "No matrix name found",
                                                  "No matrix name found",
                                                  JOptionPane.ERROR_MESSAGE);
                    return hasp0;
                  }
                }
                else if (first.equals("Vector")) {
                  if (tk.hasMoreTokens()) {
                    String next = tk.nextToken();
                    if (next.equals("R")) {
                      rfile.println("Vector");
                      active = rfile;
                    }
                    if (next.equals("S")) {
                      sfile.println("Vector");
                      active = sfile;
                    }
                    else if (next.equals("P0")) {
                      ifile.println("Vector");
                      active = ifile;
                      hasp0 = true;
                    }
                    else {
                      JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                    "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else {
                qfile.println(line);
                rfile.println(line);
                sfile.println(line);
                ifile.println(line);
              }
          }
          else {
              if (first.equals("Matrix")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("Q")) {
                    qfile.println("Matrix");
                    active = qfile;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal matrix id: "+next,
                                                  "Illegal matrix id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No matrix name found",
                                                "No matrix name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else if (first.equals("Vector")) {
                if (tk.hasMoreTokens()) {
                  String next = tk.nextToken();
                  if (next.equals("R")) {
                    rfile.println("Vector");
                    active = rfile;
                  }
                  else if (next.equals("S")) {
                    sfile.println("Vector");
                    active = sfile;
                  }
                  else if (next.equals("P0")) {
                    ifile.println("Vector");
                    active = ifile;
                    hasp0 = true;
                  }
                  else {
                    JOptionPane.showMessageDialog(this, "Illegal vector id: "+next,
                                                  "Illegal vector id: "+next,
                                                  JOptionPane.ERROR_MESSAGE);
                  }
                }
                else {
                  JOptionPane.showMessageDialog(this, "No vector name found",
                                                "No vector name found",
                                                JOptionPane.ERROR_MESSAGE);
                  return hasp0;
                }
              }
              else {
                active.println(line);
              }
            }
          }
          catch (NoSuchElementException ex) {}
        line = reader.readLine();
      }
      qfile.close();
      rfile.close();
      sfile.close();
      ifile.close();
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error writing tmp files", JOptionPane.ERROR_MESSAGE);
    }
    return hasp0;
  }

  public MRMParameterSet getParameterSet () {
    MRMParameterSet ps = new MRMParameterSet ();
    if (isRaw)
      return ps;
    else {
      extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/s.v",tmpdir+"/p0.v",null);
      Parameter p;
      try {
        Process process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir +
            "/q.m " + tmpdir + "/q.compiled");
        System.out.println("bin/compilerule " + tmpdir + "/q.m " + tmpdir +
                           "/q.compiled");
        int exitVal = process.waitFor();
        if (exitVal > 0)
          return ps;
        try {
          BufferedReader reader = new BufferedReader (new FileReader(tmpdir+"/q.compiled"));
          String delim = new String (" \n\r\t,;:(){}[]_><=");
          String line = reader.readLine();
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, delim);
            String first = tk.nextToken();
            if (!first.equals("Const"))
              break;
            String id = tk.nextToken();
            double val = Double.parseDouble(tk.nextToken());
            p = new Parameter (id, val);
            ps.parameters.add(p);
            line = reader.readLine();
          }
          reader.close();
        }
        catch (IOException e) {
          JOptionPane.showMessageDialog(this, e.toString(),"Tmp file 'q.compiled' not found!", JOptionPane.ERROR_MESSAGE);
        }
        catch (NoSuchElementException x) {
          JOptionPane.showMessageDialog(this, x.toString(),"Tmp file 'q.compiled' has bad format!", JOptionPane.ERROR_MESSAGE);
        }
      }
      catch (IOException ex) {
        JOptionPane.showMessageDialog(this, ex.toString(),"compilefrmq not found!", JOptionPane.ERROR_MESSAGE);
      }
      catch (InterruptedException ex2) {
        JOptionPane.showMessageDialog(this, ex2.toString(),"compilefrmq interrupted!", JOptionPane.ERROR_MESSAGE);
      }
      return ps;
    }

  }

  public void getARMoments (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (running.equalsIgnoreCase("Time")) {
      MRMParameterSet mp = new MRMParameterSet (p);
      mp.delParameter("Time");
      runAnalysis(mp, running, start, end, steps, numOfMoms, normalized);
    }
    else
      runPAnalysis (p, running, start, end, steps, numOfMoms, normalized);
    p.delParameter(running);
  }

  public String checkModel () {
    if (isRaw)
      return checkRaw ();
    else {
      // check type
      String mnemonic = null;
      try {
        BufferedReader file = new BufferedReader(new StringReader(modelDef.getText()));
        String delim = new String (" \n\r\t,;:(){}[]-_><=");
        String line = file.readLine();
        while (line != null) {
          StringTokenizer tk = new StringTokenizer (line, delim);
          if (tk.hasMoreTokens()) {
            if (tk.nextToken().equals("type")) {
              mnemonic = tk.nextToken();
            }
            break;
          }
          line = file.readLine();
        }
      }
      catch (IOException ex) {}
      if (mnemonic==null) {
        return "Type specification missing,\nplease insert 'type = so;' line at the beginning of the file!";
      }
      else if (!mnemonic.equalsIgnoreCase(type.getMnemonic())) {
        return "Bad type specification ("+mnemonic+"),\nplease insert 'type = so;' line at the beginning of the file!\n(type cannot be changed after openeing the file)";
      }

      // check remaining description
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v", tmpdir+"/s.v",tmpdir + "/p0.v", null);
      Parameter p;
      try {
        Process process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/q.m " + tmpdir + "/q.compiled");
        int iq = process.waitFor();
        process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/r.v " + tmpdir + "/r.compiled");
        int ir = process.waitFor();
        process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/s.v " + tmpdir + "/s.compiled");
        int is = process.waitFor();
        int ip0 = 0;
        if (hasp0) {
          process = Runtime.getRuntime().exec("bin/compilerule " + tmpdir + "/p0.v " + tmpdir + "/p0.compiled");
          ip0 = process.waitFor();
        }
        String error = null;
        if (iq!=0) {
          error = "Error in Q matrix definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/q.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        else if (ir!=0) {
          error = "Error in R vector definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/r.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        else if (is!=0) {
          error = "Error in S vector definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/s.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        else if (hasp0 && ip0!=0) {
          error = "Error in p0 vector definition!\n";
          BufferedReader reader = new BufferedReader(new FileReader(tmpdir + "/p0.compiled"));
          String line = reader.readLine();
          while (line!=null) {
            error += line + "\n";
            line = reader.readLine();
          }
          reader.close();
        }
        return error;
      }
      catch (InterruptedException ex) {
      }
      catch (IOException ex) {

      }
    }
    return null;
  }

  public void showStateSpace () {

    String commandLine = "bin/drawstates ";
    if (!isRaw) {
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v",
                                         tmpdir + "/s.v",
                                         tmpdir + "/p0.v", null);
      commandLine += "-formalizm "+tmpdir+"/q.m -o "+tmpdir+"/states.gif -t "+tmpdir+"/states.dot";
    }
    else {
      save(tmpdir + "/model.rrm");
      commandLine += "-raw " + tmpdir + "/model.rrm -o " + tmpdir + "/states.gif -t " + tmpdir + "/states.dot";
    }

    System.out.println(commandLine);

    DotRunner dr = new DotRunner (mw, commandLine, tmpdir+"/states.gif");
    dr.runDot();
  }


  public void runAnalysis (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    double[][] result = new double[steps][];
    for (int i=0; i<steps; i++)
      result[i] = new double [numOfMoms+3];

    String commandline = "";
    analyserProcess = null;
    String norm = "";
    if (normalized)
      norm = "-normalized ";

    if (isRaw) {
      save (tmpdir+"/model.rrm");
      commandline = "bin/sorm -raw " + tmpdir + "/model.rrm " +
          "-o " + tmpdir + "/res.txt" + norm + " -numofmoms " + numOfMoms +
          " -time from=" + start + " to=" + end + " steps=" + steps;
    }
    else {
      boolean hasp0 = extractDescription(tmpdir + "/q.m", tmpdir + "/r.v",
                                         tmpdir + "/s.v",
                                         tmpdir + "/p0.v", p);
      String p0 = "";
      if (hasp0)
        p0 = "P0=" + tmpdir + "/p0.v ";

      commandline = "bin/sorm -formalizm Q=" + tmpdir + "/q.m R=" +
          tmpdir + "/r.v S=" + tmpdir + "/s.v " + p0 +
          "-o " + tmpdir + "/res.txt" + norm + " -numofmoms " + numOfMoms +
          " -time from=" + start + " to=" + end + " steps=" + steps;
    }


        System.out.println(commandline);
        CommandRunner cmd = new CommandRunner (commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start ();
        String msg = "";
        int overalltotal = 6;
        String overall = "";
        String subtask = "";
        if (isRaw) {
          subtask = "Reading Input Matrices and Vectors...";
          overalltotal = 1;
        }
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);

            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            int G = 0;
            int overallcompleted = -1;
            int subtasktotal = 0;
            int subtaskcompleted = 0;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                  subtasktotal = states;
                }
                else if (line.startsWith("  Generating elements...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Generating elements";
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  overallcompleted++;
                  overall = "Overall progress: "+(overallcompleted)+"/"+overalltotal;
                  subtask = "Performing Markov reward model analysis";
                  subtasktotal = 1;
                  subtaskcompleted = 0;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  subtaskcompleted = st;
                }
                else if (line.startsWith(" G=")) {
                  G = Integer.parseInt(line.substring(3));
                  subtasktotal = G;
                }
                else if (line.startsWith("  g=")) {
                  int g = Integer.parseInt(line.substring(4));
                  subtaskcompleted = g;
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus (overall, overalltotal, overallcompleted, subtask, subtasktotal, subtaskcompleted, msg);
          }
          catch (InterruptedException ex2) {}
        }
        listener.updateStatus ("Overall progress: "+overalltotal+"/"+overalltotal,overalltotal,overalltotal,"Analysis ready",1,1,msg);
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          int linenum = 0;
          while (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            for (int i=0; i<numOfMoms+1; i++) {
              result[linenum][i] = Double.parseDouble(tk.nextToken());
              result[linenum][numOfMoms+1] = 0;
              result[linenum][numOfMoms+2] = 1e200;
            }
            linenum++;
            line = resreader.readLine();
          }
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
        listener.addResults(result);
        listener.finish ();
        analyserProcess = null;

  }

  public void runPAnalysis (MRMParameterSet p, String running, double start, double end, int steps, int numOfMoms, boolean normalized) {

    if (isRaw) {
      analyserProcess = null;
    }
    else {
      double[][] result = new double[1][];
      result[0] = new double [numOfMoms+3];
      for (int j=0; j<numOfMoms+3; j++)
        result[0][j] = 0.0;

      String norm = "";
      if (normalized)
        norm = "-normalized ";
      double nvar = p.getParameter("Time").value;
      boolean hasp0 = extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/s.v",tmpdir+"/p0.v",null);
      String p0 = "";
      if (hasp0)
        p0 = " P0="+tmpdir+"/p0.v";
      String commandline = "bin/sorm -formalizm Q="+tmpdir+"/q.m R="+tmpdir+"/r.v S="+tmpdir+"/s.v "+p0+" -o "+tmpdir+"/res.txt"+norm+" -numofmoms "+numOfMoms+" -time "+nvar;

//        commandline = "bin/mrmsolve.exe "+tmpdir+"/q.m "+tmpdir+"/r.v "+p0+tmpdir+"/res.txt " + numOfMoms + " AR " +            norm + " " + nvar + " " + nvar + " 1";
      System.out.println(commandline);

      for (int s=0; s<steps; s++) {
        double cval = start + s * (end-start) / (steps-1);

        MRMParameterSet cps = new MRMParameterSet (p);
        cps.delParameter("Time");
        cps.getParameter(running).value = cval;
        String overall = running+" = "+cval+" ("+(s+1)+"/"+steps+")";

        extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/s.v",tmpdir+"/p0.v",cps);

        CommandRunner cmd = new CommandRunner(commandline);
        cmd.start();
        while (!cmd.running) {}
        listener.start();
        String msg = "";
        int stages = 6;
        if (isRaw) {
          stages = 1;
        }
        while (cmd.running) {
          try {
            analyserProcess = cmd;
            Thread.currentThread().sleep(500);
            if (cmd.errorGobbler != null)
              msg = cmd.errorGobbler.s;

            BufferedReader reader = new BufferedReader(new StringReader(msg));
            double perc = 0;
            String subtask = "";
            int stage = 0;
            int G = 1;
            int states = 1;

            try {
              String line = reader.readLine();
              while (line != null) {
                if (line.startsWith("  #States=")) {
                  states = Integer.parseInt(line.substring(10));
                }
                else if (line.startsWith("  Generating elements...")) {
                  stage++;
                  subtask = "Generating elements";
                  perc = (stage - 1) / stages;
                }
                else if (line.startsWith("Starting Markov reward model analysis...")) {
                  stage++;
                  subtask = "Performing Markov reward model analysis";
                  perc = (stage - 1) / stages;
                }
                else if (line.startsWith("    #Processed states =")) {
                  int st = Integer.parseInt(line.substring(24));
                  perc = (stage - 1 + (double)st / states) / stages;
                }
                else if (line.startsWith(" G=")) {
                  G = Integer.parseInt(line.substring(3));
                }
                else if (line.startsWith("  g=")) {
                  int g = Integer.parseInt(line.substring(4));
                  perc = (stage - 1 + (double) g / G) / stages;
                }
                line = reader.readLine();
              }
            }
            catch (IOException ex) {
              ex.printStackTrace();
            }
            listener.updateStatus(overall, steps, s, subtask, 1000, (int)(perc*1000), msg);
          }
          catch (InterruptedException ex2) { return; }
        }
        listener.updateStatus (overall, steps, s, "Analysis ready", 1, 1, msg);
        if (cmd.aborted)
          break;
        try {
          BufferedReader resreader = new BufferedReader (new InputStreamReader (new FileInputStream (tmpdir+"/res.txt")));
          String line = resreader.readLine();
          if (line!=null) {
            StringTokenizer tk = new StringTokenizer (line, " \t\n");
            for (int i=0; i<numOfMoms+1; i++)
              result[0][i] = Double.parseDouble(tk.nextToken());
          }
          result[0][0] = cval;
          result[0][numOfMoms+1] = 0;
          result[0][numOfMoms+2] = 1e200;
          listener.addResults(result);
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this, ex.toString(),"Could not open analysis result file (res.txt)!", JOptionPane.ERROR_MESSAGE);
        }
      }
      listener.finish ();
      analyserProcess = null;
    }
  }

  public String getStateSpaceHTMLString () {

    extractDescription(tmpdir+"/q.m",tmpdir+"/r.v",tmpdir+"/s.v",tmpdir+"/p0.v",null);
    try {
      Process process = Runtime.getRuntime().exec("bin/statespacegeneration "+tmpdir+"/q.m "+tmpdir+"/states.txt");
      process.waitFor ();
      BufferedReader reader = new BufferedReader (new FileReader (tmpdir+"/states.txt"));

      StringBuffer stateSpace = new StringBuffer("<HTML> <BODY bgcolor=%BG%> <H3 align=center>State Space of "+shortname+"</H3><HR>");
      stateSpace.append("<TABLE border=0 width=100% cellspacing=0><TR><TD><TABLE width=100% border=0 cellspacing=0>");
      String color;
      boolean even = true;
      String line = reader.readLine();
      String evenPrefix = "<TR bgcolor=%TABLE1%><TD align=center>";
      String oddPrefix = "<TR bgcolor=%TABLE2%><TD align=center>";
      String postfix = "</TD></TR>";
      while (line!=null) {
        if (even)
          stateSpace.append(evenPrefix);
        else
          stateSpace.append(oddPrefix);
        stateSpace.append(line);
        stateSpace.append(postfix);
        line = reader.readLine();
        even = !even;
      }
      stateSpace.append("</TABLE></TR></TD></TABLE></BODY></HTML>");
      return stateSpace.toString();
    }
    catch (InterruptedException ex) {
      JOptionPane.showMessageDialog(this, ex.toString(),"Error during execution of statespace generation!", JOptionPane.ERROR_MESSAGE);
    }
    catch (IOException ex2) {
      JOptionPane.showMessageDialog(this, ex2.toString(),"Could not open state space file (states.txt)!", JOptionPane.ERROR_MESSAGE);
    }

    return "";
  }

}

