import javax.swing.table.AbstractTableModel;
import java.util.*;
import javax.swing.*;
import java.io.*;

public class UMatrix extends AbstractTableModel {

  Vector from = new Vector ();
  Vector to = new Vector ();
  Vector rate = new Vector ();
  int maxstate1 = 0;
  int maxstate2 = 1;
  String label = "";
  String name = "";
  boolean dirty = false;

  public UMatrix (String n, String l) {
    name = n;
    label = l;
  }

  public boolean isDirty () {
    return dirty;
  }

  public void setDirty (boolean d) {
    dirty = d;
  }

  public int getColumnCount() { return 3; }

  public int getRowCount() { return from.size();}

/*    public Class getColumnClass(int c) {
    if (c==0 || c==1)
      return Integer.TYPE;
    else
      return Double.TYPE;
  }
*/

  public String check () {

    Iterator i = rate.iterator();
    Iterator j = from.iterator();
    Iterator k = to.iterator();
    while (i.hasNext()) {
      double r = ((Double)(i.next())).doubleValue();
      int f = ((Integer)(j.next())).intValue();
      int t = ((Integer)(k.next())).intValue();
      if (f==t)
        return name+": self-transitions are not allowed! (from "+f+" to "+t+")";
      if (r<0)
        return name+": negative rates are not allowed! (from "+f+" to "+t+")";
    }
    return null;
  }

  public Object getValueAt(int row, int col) {
    if (col==0)
      return from.elementAt(row);
    else if (col==1)
      return to.elementAt(row);
    else if (col==2)
      return rate.elementAt(row);
    return null;
  }

  public void setValueAt(Object val, int row, int col) {
    if (col==0) {
      int ival = Integer.parseInt(val.toString());
      from.setElementAt(new Integer(ival), row);
      if (ival>maxstate1)
        maxstate1 = ival;
    }
    else if (col==1) {
      int ival = Integer.parseInt(val.toString());
      to.setElementAt(new Integer(Integer.parseInt(val.toString())), row);
      if (ival>maxstate2)
        maxstate2 = ival;
    }
    else if (col==2)
      rate.setElementAt( new Double (Double.parseDouble(val.toString())), row);
    dirty = true;
  }

  public String getColumnName (int i) {
    if (i==0)
      return "From";
    else if (i==1)
      return "To";
    else if (i==2)
      return label;
    return null;
  }

  public void clear () {
    from.clear();
    to.clear();
    rate.clear();
    dirty = true;
  }

  public void addLine () {
    from.add(new Integer (maxstate1++));
    to.add(new Integer (maxstate2++));
    rate.add(new Double (1.0));
    dirty = true;
  }

  public void addLine (int fr, int t, double r) {
    from.add(new Integer (fr));
    to.add(new Integer (t));
    rate.add(new Double (r));
    dirty = true;
  }

  public void delLine (int i) {
    if (i>=0 && i<from.size ()) {
      from.remove(i);
      to.remove(i);
      rate.remove(i);
      dirty = true;
    }
  }

  public boolean isCellEditable (int r, int c) {
    return true;
  }

  public void load () {

    if (getRowCount()>0 && JOptionPane.showConfirmDialog(null, "When reading a file, the current matrix content will be lost!\n Are you sure to continue?")!=JOptionPane.YES_OPTION)
      return;

    clear();

    JFileChooser chooser = new JFileChooser();
    int returnVal = chooser.showOpenDialog (null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      String delim = new String (" \n\r\t,;:(){}[]-_><=");
      try {
        String fname = chooser.getSelectedFile().getAbsolutePath();
        BufferedReader file = new BufferedReader(new FileReader(fname));
        int linenum = 1;
        String line = file.readLine();
        while (line!=null) {
          StringTokenizer tk = new StringTokenizer (line, delim);
          int toknum = 1;
          try {
            int stfrom = Integer.parseInt(tk.nextToken());
            toknum++;
            int stto   = Integer.parseInt(tk.nextToken());
            toknum++;
            double strate = Double.parseDouble(tk.nextToken());
            addLine(stfrom, stto, strate);
          }
          catch (NoSuchElementException ex1) {
            JOptionPane.showMessageDialog(null, "In line "+linenum+": "+ex1.toString(),"Error reading file", JOptionPane.ERROR_MESSAGE);
            break;
          }
          catch (NumberFormatException ex2) {
            JOptionPane.showMessageDialog(null, "In line "+linenum+", token "+toknum+": "+ex2.toString(),"Error reading file", JOptionPane.ERROR_MESSAGE);
            break;
          }
          linenum++;
          line = file.readLine();
        }
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex.toString(),"Error reading file", JOptionPane.ERROR_MESSAGE);
      }
      dirty = true;
    }
  }

}


